/*
 * $Id: AmApi.cpp,v 1.8 2004/03/08 18:56:47 rco Exp $
 *
 * Copyright (C) 2002-2003 Fhg Fokus
 *
 * This file is part of sems, a free SIP media server.
 *
 * sems is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * For a license to use the ser software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * sems is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "AmApi.h"
#include "log.h"

AmStateFactory::AmStateFactory(const string& _app_name)
  : app_name(_app_name)
{
}

int AmStateFactory::onLoad()
{
    return 0;
}

AmDialogState::AmDialogState()
    : AmEventQueue(this)
{
}

void AmDialogState::process(AmEvent* event)
{
    AmSessionEvent* session_event = dynamic_cast<AmSessionEvent*>(event);
    if(!session_event){
	ERROR("AmSession: invalid event received.\n");
	return;
    }

    DBG("in-dialog event received: %s\n",
	session_event->request.cmd.method.c_str());

    if(onOther(session_event))
	ERROR("while proceeding session event (ID=%i).\n",session_event->event_id);
}

void AmDialogState::onError(unsigned int code, const string& reason)
{
}

void AmDialogState::onBeforeCallAccept(AmRequest*)
{
}

int AmDialogState::onOther(AmSessionEvent* event)
{
    if(!event->processed){
	switch(event->event_id){
	case AmSessionEvent::Refer:
	    event->request.reply(488,"Not acceptable here");
	    break;
	case AmSessionEvent::ReInvite:
	    getSession()->negotiate(&event->request);
	    break;
	}
    }

    return 0;
}

AmStateFactory* AmDialogState::getStateFactory() 
{ 
    return state_factory; 
}

AmSession* AmDialogState::getSession() 
{ 
    return session; 
}
