/*
 * $Id: AmRequest.cpp,v 1.55 2004/06/09 09:39:16 rco Exp $
 *
 * Copyright (C) 2002-2003 Fhg Fokus
 *
 * This file is part of sems, a free SIP media server.
 *
 * sems is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * For a license to use the ser software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * sems is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "AmRequest.h"
#include "AmSession.h"
#include "AmUtils.h"
#include "AmConfig.h"
#include "AmServer.h"
#include "AmApi.h"
#include "log.h"

#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>

static int write_to_fifo(const char * buf, unsigned int len );

#define SIP_PREFIX     "sip:"
#define SIP_PREFIX_LEN (sizeof("sip:")-1)

#if 0
static int parse_uri(const string& uri, string& prefix, string& user, 
		     string& domain,string& port,string& params)
{
    string::size_type pos1,pos2;
    pos1=0;
    pos2=string::npos;
    
    if(uri.empty()){
	ERROR("passed empty uri to parse_uri.\n");
	return -1;
    }

    if(uri.substr(pos1,SIP_PREFIX_LEN) != SIP_PREFIX){
	ERROR("passed non sip uri to parse_uri ('%s')\n",uri.c_str());
	return -1;
    }
    pos1 += SIP_PREFIX_LEN;

    // [RFC 3261]
    //   userinfo =  ( user / telephone-subscriber ) [ ":" password ] "@"
    for( pos2 = pos1;; pos2++ ){

	if(pos2 >= uri.length()){
	    ERROR("parse_uri: missing '@'\n");
	    return -1;
	}

	if(uri[pos2] == '@'){
	    user = uri.substr(pos1,pos2-pos1);
	    pos1 = pos2+1;
	    break;
	}

	if(AmConfig::PrefixSep && (uri[pos2] == AmConfig::PrefixSep)){
	    prefix = uri.substr(pos1,pos2-pos1);
	    pos1 = pos2+1;
	}
    }

    // [RFC 3261]
    //   hostport =  host [ ":" port ]
    bool is_host=true;
    for( ++pos2; pos2 < uri.length(); ++pos2 ){
	switch(uri[pos2]){
	case ';':
	case '?':
	    break;
	case ':':
	    is_host = false;
	    domain = uri.substr(pos1,pos2-pos1);
	    pos1 = pos2+1;
	default:
	    continue;
	}
	break;
    }
    if(pos2>pos1){
	if(is_host)
	    domain = uri.substr(pos1,pos2-pos1);
	else
	    port = uri.substr(pos1,pos2-pos1);
	pos1 = pos2;
    }

    // parameters & headers (see RFC 3261)
    if(pos2 < uri.length())
	params = uri.substr(pos1);

    return 0;
}
#endif

AmRequest::AmRequest(const AmCmd& _cmd)
    : cmd(_cmd),replied(false),
      session_container(AmSessionContainer::instance())
{
}

int AmRequest::sendRequest(const string& method, 
			   unsigned int& code, 
			   string& reason,
			   const string& body, 
			   const string& content_type)
{
    string msg;

    string route = cmd.route;
    string res_fifo = AmSession::getNewId();
    
    replied = false;

    if (method == "CANCEL") {
	msg = ":t_uac_cancel:" + res_fifo + "\n"
	    + cmd.callid + "\n"
	    + int2str(cmd.cseq+1) + "\n\n";
    } else {	
	msg = ":t_uac_dlg:" + res_fifo + "\n"
	    + method + "\n"
	    + cmd.from_uri + "\n";
    
	if(cmd.next_hop.empty())
	    msg += ".";
	else
	    msg += cmd.next_hop;
	msg += "\n";
    
	msg += "From: " + cmd.to;
	if(cmd.to_tag.length()) msg += ";tag=" + cmd.to_tag;
	msg += "\n";
	
	msg += "To: " + cmd.from;
	if(cmd.from_tag.length()) msg += ";tag=" + cmd.from_tag;
	msg += "\n";
	
	msg += "CSeq: " + int2str(++cmd.cseq) + " " + method + "\n"
	    + "Call-ID: " + cmd.callid + "\n";
	
	msg += getContactHdr();

	// todo: do not send xtra recved headers
	if(cmd.hdrs.length()){
	    msg += cmd.hdrs;
	    if(cmd.hdrs[cmd.hdrs.length()-1] != '\n')
		msg += "\n";
	}
	
	if(!route.empty() && (route.find("Route: ")!=string::npos))
	    route = route.substr(7/*sizeof("Route: ")*/);
	
	while(!route.empty()){
	    
	    unsigned int comma_pos;
	    
	    comma_pos = route.find(',');
	    msg += "Route: " + route.substr(0,comma_pos) + "\n";
	    
	    if(comma_pos != string::npos)
		route = route.substr(comma_pos+1);
	    else
		route = "";
	}
	
	if(!body.empty())
	    msg += "Content-Type: " + content_type + "\n";
	
	msg += ".\n" // EoH
	    + body + ".\n\n";
    }  

    int ret = sendToFIFO(res_fifo,msg,code,reason);
    replied = (ret == 0);
    
    return ret;
}

int AmRequest::sendToFIFO(const string& fifo_name, const string& msg,
			  unsigned int& code, string& reason)
{
    string res_fifo = "/tmp/" + fifo_name;

    DBG("msg=<%s>\n",msg.c_str());

    FILE* reply_fifo_stream = openReplyFifo(res_fifo);
    if (reply_fifo_stream == NULL)
 	return -1;

    if(write_to_fifo(msg.c_str(),msg.length())==-1){
	fclose(reply_fifo_stream);
	return -1;
    }
     
    unsigned int res_code;
    string res_reason;

    int ret = getReturnCode(reply_fifo_stream,res_code,res_reason);

    fclose(reply_fifo_stream);
    unlink(res_fifo.c_str());

    if(ret != 0)
	ERROR("while getting return code from Ser\n");
    
    DBG("sendToFifo returns ret=getReturnCode=%i\n", ret);
    code = res_code;
    reason = res_reason;

    return ret;
}

void AmRequest::error(int code, const string& reason)
{
    if(replied)
	bye();
    else
	reply(code,reason);
}

int AmRequest::bye()
{
    unsigned int code;
    string reason;

    int ret = sendRequest("BYE",code,reason);
    if(!ret && ((code < 200) || (code >= 300))){
	ERROR("AmRequest::bye: %i %s\n",code,reason.c_str());
	return -1;
    }

    return ret;
}

string AmRequest::getContactHdr()
{
    string msg;

    if(cmd.r_uri.empty()){
	msg += "Contact: <sip:" + cmd.cmd;
	if(AmConfig::PrefixSep)
	    msg += AmConfig::PrefixSep;
	msg += "@!!>\n";
    }
    else {

      string user = cmd.user;
      if(AmConfig::PrefixSep){
	unsigned int sep_pos = cmd.user.find(AmConfig::PrefixSep);
	if(sep_pos != string::npos)
	  user = cmd.user.substr(sep_pos+1);
      }

      msg += "Contact: <sip:";
	
      if(AmConfig::PrefixSep)
	msg += cmd.cmd + AmConfig::PrefixSep;

#ifdef SUPPORT_IPV6
      if(cmd.dstip.find('.') != string::npos)
	  msg += user + "@" + cmd.dstip;
      else
	  msg += user + "@[" + cmd.dstip + "]";
#else
      msg += user + "@" + cmd.dstip;
#endif
      if(!cmd.port.empty())
	msg += ":" + cmd.port;
      msg += ">\n";
    }

    return msg;
}

void AmRequestUAS::accept(int localport, const SdpPayload& payload, 
			  SdpMedia::Direction dir)
{
    string sdp_buf;
    
    sdp.genResponse(cmd.dstip,localport,payload,dir,sdp_buf);
    if(reply(200,"OK",sdp_buf)!=0)
	throw AmSession::Exception(500,"could not send response.");
}

int AmRequestUAS::reply(int code, const string& reason, const string& body)
{
    string code_str = int2str(code);
    string res_fifo = AmSession::getNewId();

    string msg = ":vm_reply:" + res_fifo + "\n"
	+ code_str + "\n" + reason + "\n" + cmd.key + "\n" + cmd.to_tag + "\n";

    msg += getContactHdr();

    if(!body.empty())
	msg += "Content-Type: application/sdp\n";

    msg += ".\n";

    if(!body.empty())
	msg += body;

    msg += ".\n\n";

    unsigned int res_code;
    string res_reason;
    int ret = sendToFIFO(res_fifo,msg,res_code,res_reason);
    if(!ret && ((res_code < 200) || (res_code >= 300))){
	ERROR("AmRequestUAS::reply: %i %s\n",res_code,res_reason.c_str());
	return -1;
    }

    if(code >= 200)
	replied = true;

    return ret;
}

/* creates a reply fifo, opens it nonblocking, then sets flags to 
 * blocking returns NULL on failure, reading must be retried anyway! */
FILE* AmRequest::openReplyFifo(string reply_fifo_filename) {

    FILE* reply_fifo_stream;
 
    if( (mkfifo(reply_fifo_filename.c_str(),FIFO_PERM)<0) && (errno!=EEXIST) ) {
	ERROR("while creating fifo `%s': %s \n",
	      reply_fifo_filename.c_str(),strerror(errno));
	return NULL;
    }
    
    struct stat filestat;
    if (stat(reply_fifo_filename.c_str(), &filestat)==-1) { /* FIFO doesn't exist yet ... */
	return NULL;
    } else { /* file can be stat-ed, check if it is really a FIFO */
	if (!(S_ISFIFO(filestat.st_mode))) {
	    ERROR("ERROR: the file is not a FIFO: %s\n", reply_fifo_filename.c_str() );
	    return NULL;
	}
    }
    
    /* filehandle for open, being used by fdopen and therefor 
     * does not have to be closed if stream gets closed  */
    int reply_filehandle;
    
    reply_filehandle = open (reply_fifo_filename.c_str(), O_RDONLY | O_NONBLOCK , 0);
    if (reply_filehandle == -1) {
	ERROR("could not open reply_fifo, reason %s\n", strerror(errno));
	return NULL;
    }	
    
    /* now set blocking mode to make shure answer is read */
    int flags;
    if ( (flags=fcntl(reply_filehandle, F_GETFL, 0))<0) {
	ERROR("getfl failed: %s\n", strerror(errno));
	goto error;
    }
    flags&=~O_NONBLOCK;
    if (fcntl(reply_filehandle, F_SETFL, flags)<0) {
	ERROR("setfl cntl failed: %s\n", strerror(errno));
	goto error;
    }
    
    /* will be closed somewhere else */
    if( !(reply_fifo_stream = fdopen(reply_filehandle,"r")) ) {
	ERROR("while opening fifo `%s': %s\n",reply_fifo_filename.c_str(),strerror(errno));
	goto error;
    }
    return reply_fifo_stream;

 error:
    close(reply_filehandle);
    return NULL;
}

#define sleep_us(nusecs) \
	{ \
	struct timeval tval; \
	tval.tv_sec=nusecs/100000; \
	tval.tv_usec=nusecs%1000000; \
	select(0, NULL, NULL, NULL, &tval ); \
	}

#define REPLYFIFO_READ_RETRY 10*60*5

int AmRequest::getReturnCode(FILE* fifo_stream,
     unsigned int& code, string& reason) {

    char res_code_str[4] = {'\0'};
    char lbuff[MAX_LINE_SIZE];

    int retry = REPLYFIFO_READ_RETRY;
    
    while (1) {
	
	if( fifo_get_line(fifo_stream,lbuff,MAX_LINE_SIZE) == -1 ){
	    ERROR("while receiving return code from Ser: buffer overflow !\n");
	    goto error;
	}
	
	if (strlen(lbuff) == 0) {
	    if (retry-- == 0) {
		ERROR("waiting for fifo reply from ser, timeout\n");
		goto error;
	    }	
	    /* fifo not ready */
	    //DBG("sleeping and retrying to read answer from ser\n");
	    sleep_us(100000);
	    continue;
	}	
	break;
    }
    
    DBG("response from Ser: %s\n",lbuff);
    if( parse_return_code(lbuff,res_code_str,code,reason) == -1 ){
	ERROR("while parsing return code from Ser.\n");
	goto error;
    }

    if(code >= 200 && code < 300){

        AmCmd n_cmd;
	char   body[MAX_BODY_SIZE];
	
	/* Parse complete response:
	 *
	 *   [next_request_uri->cmd.from_uri]CRLF
	 *   [next_hop->cmd.next_hop]CRLF
	 *   [route->cmd.route]CRLF
	 *   ([headers->n_cmd.hdrs]CRLF)*
	 *   CRLF
	 *   ([body->body]CRLF)*
	 */
	
	READ_FIFO_PARAMETER(fifo_stream,n_cmd.from_uri,lbuff,MAX_LINE_SIZE);
	if(!n_cmd.from_uri.empty()) cmd.from_uri = n_cmd.from_uri;

	READ_FIFO_PARAMETER(fifo_stream,n_cmd.next_hop,lbuff,MAX_LINE_SIZE);
	if(!n_cmd.next_hop.empty()) cmd.next_hop = n_cmd.next_hop;

	READ_FIFO_PARAMETER(fifo_stream,n_cmd.route,lbuff,MAX_LINE_SIZE);
	if(!n_cmd.route.empty()) cmd.route = n_cmd.route;

	int ret;
	while( (ret = fifo_get_line(fifo_stream,lbuff,MAX_LINE_SIZE)) > 0 ){
	    DBG("header: %s\n",lbuff);
	    n_cmd.hdrs += string(lbuff) + "\n";
	}

	if(!n_cmd.hdrs.empty()){

	    cmd.from = n_cmd.getHeader("To");
	    DBG("found 'To: %s'\n",cmd.from.c_str());

	    unsigned int p = cmd.from.find(";tag=");
	    if(p != string::npos){
		p += 5/*sizeof(";tag=")*/;
		unsigned int p_end = p;
		while(p_end < cmd.from.length()){
		    if( cmd.from[p_end] == '>'
			|| cmd.from[p_end] == ';' )
			break;
		    p_end++;
		}
		cmd.from_tag = cmd.from.substr(p,p_end-p);
		DBG("found To-tag: '%s'\n",cmd.from_tag.c_str());
	    }

	    cmd.callid = n_cmd.getHeader("Call-ID");

	    if(!ret)
		ret = fifo_get_lines(fifo_stream,body,MAX_BODY_SIZE);
	}

	if(body[0]!='\0'){
	    m_body = body;
	}
    }

    return 0;

 error:
    return -1;
}

AmRequestUAS::AmRequestUAS(const AmCmd& _cmd, char* body_buf)
    : AmRequest(_cmd) 
{
    if(body_buf){
	m_body = body_buf;
    }
}

AmRequest* AmRequestUAS::duplicate()
{
    return new AmRequestUAS(*this);
}

void AmRequestUAS::execute()
{
    string hash_str = cmd.callid;
    if(cmd.from_tag.length())
	hash_str += cmd.from_tag;

    if(cmd.method == "INVITE") {

	sdp.setBody(m_body.c_str());
	session_container->startSession(hash_str,cmd.to_tag,this);
    }
    else if(cmd.method == "BYE" || cmd.method == "CANCEL"){
	
	if(session_container->sadSession(hash_str,cmd.to_tag))
	    reply(200,"OK");
	else 
	    reply(481,"Call/Transaction Does Not Exist");
    }
    else if(cmd.method == "REFER"){
	
	if(cmd.to_tag.empty()){
	    // Out-of-dialog REFER
	    string refer_to = uri_from_name_addr(cmd.getHeader("Refer-To"));
	    AmCmd dial_cmd =
		AmRequestUAC::dialout(cmd.user,
				      cmd.cmd,
				      refer_to,
				      cmd.r_uri);

	    cmd.to_tag = AmSession::getNewId();
	    reply(202,"Accepted");
	}
	else if(!session_container->postEvent( hash_str,cmd.to_tag,
					       new AmSessionEvent(AmSessionEvent::Refer,*this) 
					       ) ) {
	    reply(481,"Call/Transaction Does Not Exist");
	}
    }
    else if(cmd.method == "NOTIFY"){
	
	if(!session_container->postEvent( hash_str,cmd.to_tag,
					  new AmSessionEvent(AmSessionEvent::Notify,*this) 
					  ) ) {
	    
	    reply(481,"Call/Transaction Does Not Exist");
	}
    }
    else
	reply(500,"Method not supported");
}

AmRequest* AmRequestUAC::duplicate()
{
    return new AmRequestUAC(*this);
}

void AmRequestUAC::execute()
{
    string hash_str = cmd.callid;
    if(cmd.to_tag.length())
	hash_str += cmd.to_tag;

    try {
	unsigned int code;
	string       reason;

	if(cmd.method == "INVITE"){

	    AmSession* session = session_container->createSession(cmd.cmd,this);
	    if(session){

		try {
		    if(cmd.dstip.empty())
			throw string("AmCmd::dstip is empty.");
		    
		    string body;
		    session->rtp_str.setLocalIP(cmd.dstip);
		    sdp.genRequest(cmd.dstip,session->rtp_str.getLocalPort(),body);
		    
		    if(sendRequest(cmd.method,code,reason,
				   body,"application/sdp") != 0)
			throw string("while sending request.");

		    if( (code < 200) || (code >= 300) ){
			session->getDialogState()->onError(code,reason);
			AmThreadWatcher::instance()->add(session);
			return;
		    }

		    if(cmd.from_tag.empty())
			throw string("no remote user tag.");
		    
		    if(cmd.callid.empty())
			throw string("no callid.");

		    if(cmd.to_tag.empty())
			throw string("no local user tag.");

		    if(!m_body.empty())
			sdp.setBody(m_body.c_str());

		    session->req.reset(this->duplicate());
		    session_container->addSession(cmd.callid+cmd.from_tag,
						  cmd.to_tag,session);
		    session->start();
		    
		}
		catch(...){
		    AmThreadWatcher::instance()->add(session);
		    throw;
		}
	    }
	}
	else {
	    sendRequest(cmd.method,code,reason);
	    if( (code < 200) || (code >= 300) )
		throw AmSession::Exception(code,reason);
	}
    }
    catch(const AmSession::Exception& e){
 	ERROR("%i %s\n",e.code,e.reason.c_str());
    }
    catch(const string& err){
	ERROR("AmRequestUAC::execute: %s\n",err.c_str());
    }
    catch(...){
	ERROR("unexpected exception\n");
		}
}

AmCmd AmRequestUAC::dialout(const string& user,
			    const string& app_name,
			    const string& uri, 
			    const string& from)
{
    AmCmd cmd;

    cmd.cmd      = app_name;
    cmd.user     = user;
    cmd.method   = "INVITE";
    cmd.dstip    = AmConfig::LocalIP;
    cmd.from_uri = uri;
    cmd.from     = cmd.from_uri;
    cmd.to       = from;
    cmd.to_tag   = AmSession::getNewId();
    cmd.cseq     = 10;
    cmd.callid   = AmSession::getNewId() + "@" + AmConfig::LocalIP;

    AmASyncRequestUAC* req = new AmASyncRequestUAC(cmd);
    req->start();
    AmThreadWatcher::instance()->add(req);
    
    return cmd;
}

AmCmd AmRequestUAC::refer(const AmCmd& cmd,
			  const string& refer_to)
{
    AmCmd m_cmd(cmd);

    m_cmd.method   = "REFER";
    m_cmd.hdrs     += "Refer-To: " + refer_to;

    AmASyncRequestUAC* req = new AmASyncRequestUAC(m_cmd);
    req->start();
    AmThreadWatcher::instance()->add(req);

    return m_cmd;
}

void AmRequestUAC::cancel(const AmCmd& cmd)
{
    AmCmd m_cmd(cmd);
    m_cmd.method = "CANCEL";

    AmASyncRequestUAC* req = new AmASyncRequestUAC(m_cmd);
    req->start();
    AmThreadWatcher::instance()->add(req);
}

AmASyncRequestUAC::AmASyncRequestUAC(const AmCmd& _cmd)
    : AmRequestUAC(_cmd)
{
}

void AmASyncRequestUAC::run()
{
    execute();
}

void AmASyncRequestUAC::on_stop()
{
}

static int write_to_fifo(const char * buf, unsigned int len )
{
    int   fd_fifo;
    int ret=0;

    for(int retry=0; retry<4; retry++){
	
	if(retry){
	    DBG("retrying in 1 s.\n");
	    sleep(1);
	}

	if((fd_fifo = open(AmConfig::SerFIFO.c_str(),
			   O_WRONLY | O_NONBLOCK)) == -1) {
	    switch(errno){
		case ENXIO:
		    ERROR("SIP Express Router (ser) is not running !\n");
		default:
		    ERROR("while opening %s: %s\n",
			  AmConfig::SerFIFO.c_str(),strerror(errno));
	    }
	    ret = -1;
	}
	else {
	    ret = 0;
	    break;
	}
    }

    if(ret)
	return ret;

    write(fd_fifo,buf,len);
    close(fd_fifo);

    DBG("Write to fifo: completed\n");

    return 1;
}
