/*
 * $Id: AmRequest.h,v 1.17 2004/03/08 18:56:47 rco Exp $
 *
 * Copyright (C) 2002-2003 Fhg Fokus
 *
 * This file is part of sems, a free SIP media server.
 *
 * sems is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * For a license to use the ser software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * sems is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _AmRequest_h_
#define _AmRequest_h_

#include "AmCmd.h"
#include "AmSdp.h"
#include "AmThread.h"

#include <stdio.h>
#include <string>
using std::string;

class AmSession;
class AmSessionContainer;

/**
 * Holds and handles the client's request and following responses.
 * 
 */
class AmRequest
{
protected:

    AmSdp                sdp;
    AmSessionContainer*  session_container;
    bool                 replied;
    string               m_body;

    /** @return 0 if succeded. */
    int sendRequest(const string& method,
		    unsigned int& code, 
		    string& reason,
		    const string& body = "",
		    const string& content_type = "");

    /** @return 0 if succeded. */
    int sendToFIFO(const string& fifo_name, const string& msg,
		   unsigned int& code, string& reason);

    /** @return 0 if succeded. */
    //    int getReturnCode(const char* fifo_name, unsigned int& code, string& reason);
    int getReturnCode(FILE* fifo_stream, unsigned int& code, string& reason);
    /** @return NULL on failure. */
    FILE* openReplyFifo(string filename);

    string getContactHdr();

    AmRequest(const AmCmd& _cmd);

public:
    AmCmd cmd;

    bool getReplied() { return replied; }

    virtual AmRequest* duplicate()=0;
    virtual ~AmRequest(){}

    /**
     * Sends a reply.
     * @return 0 if succeded.
     */
    virtual int reply(int code, const string& reason, const string& body="") { return 0; }

    /** 
     * Accept the SDP request. 
     * Warning: throws AmSession::Exception()
     */
    virtual void accept(int localport, const SdpPayload& payload,
			SdpMedia::Direction dir) {}

    /**
     * Process the request.
     * @return true if failed
     */
    virtual void execute()=0;

    /**
     * End the dialog.
     * @return 0 if succeded.
     */
    int bye();

    /**
     * Raise an error.
     */
    void error(int code, const string& reason);
    
    friend class AmSession;
};

class AmRequestUAS: public AmRequest
{
public:
    AmRequestUAS(const AmCmd& _cmd, char* body_buf);
    AmRequest* duplicate();
    void execute();

    void accept(int localport, const SdpPayload& payload,SdpMedia::Direction dir);
    int reply(int code, const string& reason, const string& body="");

    const string& getBody() { return m_body; }
};

class AmRequestUAC: public AmRequest
{
public:
    AmRequestUAC(const AmCmd& _cmd)
	: AmRequest(_cmd) {}

    AmRequest* duplicate();

    void execute();
    
    /**
     * Dials a SIP User Agent.
     * @param user      Any string you want to get back in AmCmd.user
     *                  within the following call to AmStateFactory::onInvite.
     * @param app_name  Name of the application plug-in which gets the call.
     * @param uri       Client's uri.
     * @param from_user Value for SIP 'From' header.
     * @return          A copy of the AmCmd object which has used to 
     *                  constuct the request.
     */
    static AmCmd dialout(const string& user,
			 const string& app_name,
			 const string& uri, 
			 const string& from_user);

    /**
     * Sends an in-dialog REFER.
     * @param _cmd A copy of the AmCmd object which has
     *             been used to construct the request which
     *             has to be REFER-ed.
     */
    
    static AmCmd refer(const AmCmd& cmd,
		       const string& refer_to);

    /**
     * Cancels a previous Request.
     * @param _cmd A copy of the AmCmd object which has
     *             been used to construct the request which
     *             has to be canceled.
     */
    static void cancel(const AmCmd& cmd);
};

class AmASyncRequestUAC: public AmRequestUAC, public AmThread
{
public:
    AmASyncRequestUAC(const AmCmd& _cmd);
    void run();
    void on_stop();
};

#endif

// Local Variables:
// mode:C++
// End:
