#include "AmRtpPacketTracer.h"
#include "AmRtpPacket.h"
#include "log.h"


#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <errno.h>

AmRtpPacketTracer::AmRtpPacketTracer(const string& trace_file)
{
    trace_fd = open(trace_file.c_str(),O_WRONLY|O_CREAT|O_TRUNC, DEFFILEMODE/*0666*/);
    if(trace_fd<0){
	DBG("RTP packet tracer: could not open trace file (%s): %s\n",
		trace_file.c_str(),strerror(errno));
    }
    DBG("*** RTP tracing is enabled ***\n");
    DBG("*** trace file @ %s ***\n",trace_file.c_str());
}

AmRtpPacketTracer::~AmRtpPacketTracer()
{
    if(trace_fd>=0)
	close(trace_fd);
}

void AmRtpPacketTracer::write_packet(unsigned int ts,
#ifdef SUPPORT_IPV6
				     struct sockaddr_storage* recv_addr,
#else
				     struct sockaddr_in* recv_addr,
#endif
				     AmRtpPacket* rp)
{
    if(trace_fd<0)
	return;

    unsigned int s = sizeof(unsigned int) 
	+ sizeof(struct timeval) 
	+ sizeof(struct sockaddr_in) 
	+ rp->b_size;

    struct timeval now;
    gettimeofday(&now,NULL);

    wm.lock();
    write(trace_fd,&s,sizeof(unsigned int));
    write(trace_fd,&ts,sizeof(unsigned int));
    write(trace_fd,&now,sizeof(struct timeval));
#ifdef SUPPORT_IPV6
    write(trace_fd,recv_addr,sizeof(struct sockaddr_storage));
#else
    write(trace_fd,recv_addr,sizeof(struct sockaddr_in));
#endif
    write(trace_fd,rp->buffer,rp->b_size);
    wm.unlock();
}
