/*
 * $Id: AmSmtpClient.h,v 1.4 2003/03/14 12:16:43 rco Exp $
 *
 * Copyright (C) 2002-2003 Fhg Fokus
 *
 * This file is part of sems, a free SIP media server.
 *
 * sems is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * For a license to use the ser software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * sems is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _AmSmtpClient_h_
#define _AmSmtpClient_h_

#include <string>
#include <vector>
using std::string;
using std::vector;

/** 
 * SMTP Line buffer for commands and responses 
 * (not for data transmission!) 
 */
#define SMTP_LINE_BUFFER  512

/**
 * Email file attachement
 */
struct Attachement
{
    /** Local file name */
    string fullname;
    
    /** Proposed remote file name */
    string filename; 
    /** Declared content type */
    string content_type;

    Attachement(const string& _full, const string& _file="", const string& _ct="")
	: fullname(_full), filename(_file), content_type(_ct) {}
};

typedef vector<Attachement> Attachements;

/**
 * SMTP client implementation class
 * 
 */
class AmSmtpClient
{
    string         server_ip;
    unsigned short server_port;

    /** socket descriptor */
    int          sd;
    /** size of last response receved */
    unsigned int received;
    /** recv & scratch buffer */
    char         lbuf[SMTP_LINE_BUFFER];
    /** code of the last response */
    unsigned int res_code;
    /** code of the last response */
    char         res_code_str[4]; // null-terminated
    /** textof the last response */
    string       res_msg;

    enum Status { st_None=0, st_Ok, st_Error, st_Unknown };
    /** Client status */
    Status status;

    /** @return true if failed */
    bool read_line();
    /** @return true if failed */
    bool get_response();
    /** @return true if failed */
    bool parse_response();
    /** @return true if failed */
    bool send_line(const string& cmd);
    /** @return true if failed */
    bool send_data(const string& body, const Attachements& attachements);

 public:
    /** Additional header fields */
    vector<string> headers;
    /** Char set */
    string charset;

    AmSmtpClient(const vector<string>& _headers);
    ~AmSmtpClient();

    /** @return true if failed */
    bool connect(const string& _server_ip, unsigned short _server_port);
    /** @return true if failed */
    bool send_command(const string& cmd);
    /** @return true if failed */
    bool send_body(const string& body, const Attachements& attachements); 
    /** @return true if failed */
    bool close();
};

#endif

// Local Variables:
// mode:C++
// End:

