/*
 * $Id: AmThread.cpp,v 1.7 2004/02/28 12:47:29 rco Exp $
 *
 * Copyright (C) 2002-2003 Fhg Fokus
 *
 * This file is part of sems, a free SIP media server.
 *
 * sems is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * For a license to use the ser software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * sems is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "AmThread.h"
#include "log.h"

#include <unistd.h>

AmMutex::AmMutex() 
{
    pthread_mutex_init(&m,NULL);
}

AmMutex::~AmMutex()
{
    pthread_mutex_destroy(&m);
}

void AmMutex::lock() 
{
    pthread_mutex_lock(&m);
}

void AmMutex::unlock() 
{
    pthread_mutex_unlock(&m);
}

AmThread::AmThread()
    : _stopped(true)
{
}

void * AmThread::_start(void * _t)
{
    AmThread* _this = (AmThread*)_t;
    _this->_stopped.set(false);
    _this->run();
    _this->_stopped.set(true);
    
    return NULL;
}

void AmThread::start()
{
		int res;
    res = pthread_create(&_td,NULL,_start,this);
		if (res != 0) {
	ERROR("pthread create failed with code %i\n", res);
		}	
}

void AmThread::stop()
{
    _m_td.lock();

    // gives the thread a chance to clean up
    try { on_stop(); } catch(...) {}

    int res;
    if ((res = pthread_detach(_td)) != 0) {
			ERROR("pthread_detach failed with code %i\n", res);
		};
    //pthread_cancel(_td);

    _m_td.unlock();
}

void AmThread::join()
{
    if(!getStopped())
	pthread_join(_td,NULL);
}


AmThreadWatcher* AmThreadWatcher::_instance=0;
AmMutex AmThreadWatcher::_inst_mut;

AmThreadWatcher::AmThreadWatcher()
    : _run_cond(false)
{
}

AmThreadWatcher* AmThreadWatcher::instance()
{
    _inst_mut.lock();
    if(!_instance){
	_instance = new AmThreadWatcher();
	_instance->start();
    }

    _inst_mut.unlock();
    return _instance;
}

void AmThreadWatcher::add(AmThread* t)
{
    DBG("trying to add thread.\n");
    q_mut.lock();
    thread_queue.push(t);
    _run_cond.set(true);
    q_mut.unlock();
    DBG("thread added.\n");
}

void AmThreadWatcher::on_stop()
{
}

void AmThreadWatcher::run()
{
    for(;;){

	_run_cond.wait_for();
	// Let some time for to threads 
	// to stop by themselves
	sleep(10);

	q_mut.lock();
	DBG("Thread watcher starting its work\n");

	try {
	    queue<AmThread*> n_thread_queue;

	    while(!thread_queue.empty()){

	        AmThread* cur_thread = thread_queue.front();
		thread_queue.pop();

		q_mut.unlock();

		if(cur_thread->getStopped()){
		    delete cur_thread;
		    DBG("thread %x has been destroyed'\n",(unsigned int)cur_thread);
		}
		else {
		    DBG("thread %x still running\n",(unsigned int)cur_thread);
		    n_thread_queue.push(cur_thread);
		}

		q_mut.lock();
	    }

	    swap(thread_queue,n_thread_queue);

	}catch(...){
		/* this one is IMHO very important, as lock is called in try block! */
		ERROR("unexpected exception, state may be invalid!\n");
	}

	bool more = !thread_queue.empty();
	q_mut.unlock();

	DBG("Thread watcher finished\n");
	if(!more)
	    _run_cond.set(false);
    }
}

