/*
 * $Id: Conference.cpp,v 1.6 2004/01/20 10:50:42 rco Exp $
 *
 * Copyright (C) 2002-2003 Fhg Fokus
 *
 * This file is part of sems, a free SIP media server.
 *
 * sems is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * For a license to use the sems software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * sems is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "Conference.h"
#include "ConferenceStatus.h"
#include "AmUtils.h"
#include "SemsConfiguration.h"

#include "sems.h"
#include "log.h"

#define APP_NAME "conference"

EXPORT_FACTORY(ConferenceFactory,APP_NAME);

ConferenceFactory::ConferenceFactory(const string& _app_name)
    : AmStateFactory(_app_name)
{
    status_container = ConferenceStatusContainer::instance();
}

string ConferenceFactory::LonelyUserFile;
string ConferenceFactory::TracePath;

int ConferenceFactory::onLoad()
{
    char* p = 0;

    SemsConfiguration m_config;

    if(m_config.reloadModuleConfig(APP_NAME) != 1)
	return -1;

    if( ((p = m_config.getValueForKey("default_announce")) != NULL) && (*p != '\0') )
	LonelyUserFile = p;
    else {
	WARN("no default_announce specified in configuration\n");
	WARN("file for module conference.\n");
	LonelyUserFile = ANNOUNCE_PATH "/" ANNOUNCE_FILE;
    }

    if(!file_exists(LonelyUserFile)){
	ERROR("default announce '%s' \n",LonelyUserFile.c_str());
	ERROR("for module conference does not exist.\n");
	return -1;
    }

    if( ((p = m_config.getValueForKey("trace_path")) != NULL) && (*p != '\0') ){
	TracePath = p;
	INFO("Conference tracer enabled: files will be written in '%s'\n",p);
    }

    return 0;
}

AmDialogState* ConferenceFactory::onInvite(AmCmd& cmd)
{
    string conf_id = cmd.user;

    bool status_is_new = status_container->createStatus(conf_id);
    if(status_is_new){
	// user wants to create a conference
	DBG("User '%s' wants to create a conference.\n",cmd.from.c_str());
    }
    else {
	// user wants to join an existing conference
	DBG("User '%s' wants to join an existing conference.\n",cmd.from.c_str());
    }

    cmd.r_uri = "sip:" + conf_id + "@" + cmd.dstip;

    return new ConferenceDialog(conf_id);
}

ConferenceDialog::ConferenceDialog(const string& conf_id)
    : conf_id(conf_id), unbound2status(true)
{
}

ConferenceDialog::~ConferenceDialog()
{
}

void ConferenceDialog::onSessionStart(AmRequest* req)
{
    unbound2status.set(false);

    if(ConferenceStatusContainer::instance()->addDialog(conf_id,this)){
	
	DBG("dialog running...\n");
  	while(!getSession()->sess_stopped.get()){
	    waitForEvent();
  	    processEvents();
	}
  	processEvents();

	DBG("waiting for status to unbind us...\n");
	unbound2status.wait_for();
	DBG("... unbound: onSessionStart finished\n");
    }
    else
	throw string("Conference does not exist anymore !");
}

void ConferenceDialog::onBye(AmRequest* req)
{
    ConferenceStatusContainer::instance()->removeDialog(conf_id,this);
}

int ConferenceDialog::onOther(AmSessionEvent* event)
{
    if(event->event_id == AmSessionEvent::Refer){

	AmCmd& cmd = event->request.cmd;
	string refer_to = uri_from_name_addr(cmd.getHeader("Refer-To"));
	AmCmd dial_cmd =
	    AmRequestUAC::dialout(cmd.user,
				  APP_NAME,
				  refer_to,
				  cmd.r_uri);
	event->processed = true;
    }

    return AmDialogState::onOther(event);
}

void ConferenceDialog::unbind()
{
    unbound2status.set(true);
}

