/*
 * $Id: ConferenceStatus.h,v 1.4 2004/03/08 18:56:47 rco Exp $
 *
 * Copyright (C) 2002-2003 Fhg Fokus
 *
 * This file is part of sems, a free SIP media server.
 *
 * sems is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * For a license to use the sems software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * sems is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _ConferenceStatus_h_
#define _ConferenceStatus_h_

#include "AmThread.h"
#include "AmRtpStream.h"
#include "MultiPartyMixer.h"
#include "AmEventQueue.h"

#include <set>
#include <queue>
#include <string>
using std::set;
using std::pair;
using std::queue;
using std::string;

class ConferenceDialog;
class ConferenceStatusContainer;

struct DialogEvent: public AmEvent
{
    ConferenceDialog* dialog;
    DialogEvent(int action, ConferenceDialog* dialog);
};

class ConferenceStatus: public AmThread, public AmEventHandler
{
    enum Action { CSC_Remove, CSC_Insert };
    typedef DialogEvent ConferenceStatusCmd;
    
    string            conf_id;
    AmSharedVar<bool> runcond;

    AmRtpStreamSet         str_set;
    MultiPartyMixer        mixer;

    set<ConferenceDialog*> dialogs;
    AmMutex                dialogs_mut;

    AmEventQueue           request_q;

    bool                   had_lonely_user;
    ConferenceDialog*      lonely_dialog;

    void pushRequest(Action a, ConferenceDialog* cd);
    void process(AmEvent* event);

    // AmThread interface
    virtual void run();
    virtual void on_stop();

    ConferenceStatus(const string& conference_id);
    ~ConferenceStatus();

    void addDialog(ConferenceDialog* dialog);
    void removeDialog(ConferenceDialog* dialog);

 public:
    friend class ConferenceStatusContainer;
};

class ConferenceStatusContainer : public AmThread
{
    static ConferenceStatusContainer* _instance;

    map<string,ConferenceStatus*> cid2status;
    AmMutex                       cid2s_mut;

    queue<ConferenceStatus*> d_status;
    AmMutex                  ds_mut;

    /** the daemon only runs if this is true */
    AmCondition<bool> _run_cond;
    
    ConferenceStatusContainer();
    
    // AmThread interface
    virtual void run();
    virtual void on_stop();

public:
    static ConferenceStatusContainer* instance();

    /**
     * @return true if a status has been created.
     *         false if status already exists.
     */
    bool createStatus(const string& conf_id);

    void destroyStatus(const string& conf_id);

    /**
     * @return true if the request has been passed to the status.
     *         false if something went wrong.
     */
    bool addDialog(const string& conf_id, ConferenceDialog* dialog);

    void removeDialog(const string& conf_id, ConferenceDialog* dialog);
};

#endif
// Local Variables:
// mode:C++
// End:

