/*
 * $Id: AmAudioIsdn.h,v 1.2 2003/11/16 19:17:55 ullstar Exp $
 *
 * Copyright (C) 2003 Ulrich Abend
 *
 * This file is part of isdngw, a free isdn gateway plugin 
 * for sems, a free SIP media server.
 *
 * isdngw is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * For a license to use the ser software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * isdngw is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/** \file AmAudioIsdn.h Defines classes AmAudioIsdn and AmAudioIsdnFormat. 
 * Both classes are needed for successful duplex connection and 
 * transformation to and from isdn audion mode, which is currently hardcoded
 * to ulaw */
#ifndef _AmAudioIsdn_h_
#define _AmAudioIsdn_h_

#include "AmAudio.h"
#include "IsdnDevice.h"
#include "globals.h"
		
#ifdef USE_CAPI20
#include "IsdnCapiConnection.h"
#endif

#define AUDIO_TYPE_ULAW 0
#define AUDIO_TYPE_ALAW 1

/** Audio converter and access class for interaction with isdn audio data. */
class AmAudioIsdn : public AmAudio {
	private:
		/** IsdnDevice used in this connection */
		IsdnDevice *m_idev;

		#ifdef USE_CAPI20
		IsdnCapiConnection *m_ccon;
		#endif 

	public:
    AmAudioIsdn(IsdnDevice *idev, bool fromIsdn);
		#ifdef USE_CAPI20
    AmAudioIsdn(IsdnCapiConnection *ccon, bool fromIsdn);
		#endif 

    ~AmAudioIsdn();
		void close();
		int streamGet(unsigned int user_ts, unsigned int size);
		int streamPut(unsigned int user_ts, unsigned int size);
};		

/** Isdn audio format description. */
class AmAudioIsdnFormat : public AmAudioFormat {
	public:
		AmAudioIsdnFormat(int type);
		int getCodecId();
	private:
		int m_type;
};

#endif
