/*
 * $Id: IsdnCall.h,v 1.4 2004/03/29 16:29:52 ullstar Exp $
 *
 * Copyright (C) 2003 Ulrich Abend
 *
 * This file is part of isdngw, a free isdn gateway plugin 
 * for sems, a free SIP media server.
 *
 * isdngw is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * For a license to use the ser software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * isdngw is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef _ISDNCALL_H_
#define _ISDNCALL_H_

#include <pthread.h>
#include <time.h>
#include "globals.h"
#include "AmApi.h"

#define CALLSTATE_IDLE 0
#define CALLSTATE_CONNECTING 1
#define CALLSTATE_CONNECTED 2
//#define CALLSTATE_CONNECTED_AUDIO 3
#define CALLSTATE_DISCONNECTED 3

#define CONNECTIONTYPE_I4L 0
#define CONNECTIONTYPE_CAPI 1

typedef struct History {
	History* next;
	struct timeval timestamp;
	char* description;
	bool pstnConnected; /* you can calc the online time with this one */
};	

typedef struct CallDetail {
	int id; /* used by fifo web commands */
	CallDetail* next;
	char* fromnumber;
	char* tonumber;
	char* fromuri;
	char* touri;
	char* backend;
	char* device;
	char direction[10];
	History* history;
};	

class IsdnCall
{
public:
	IsdnCall();
	~IsdnCall();

	void placeCallFromPSTN(char* fromnumber, char* tonumber, int backend, void* device);
	void placeCallFromSIP(AmCmd cmd);

	void changeSIPState(int newstate, int code=0, const char* reason=NULL);
	void changePSTNState(int newstate, int code=0, const char* reason=NULL);

	void setCommand(AmCmd cmd);
	void setRequest(AmRequest* req);
	void setSession(AmSession* ses);


	void lock();
	void unlock();
	
	bool isDisconnected();

	/* functions for reporting in IsdnCallManager */
	char* getBackendAsText();
	char* getDeviceAsText();
	char* getSIPStateAsText();
	char* getPSTNStateAsText();
	char* getFromAsText();
	char* getToAsText();
	const char* getDirection();

	char* getToNumber();
	char* getFromNumber();
	char* getToSip();
	char* getFromSip();


private:

	void* m_pstndevice;
	CallDetail* m_calldetail;

	int m_sipcode;
	char m_sipreason[100];

	pthread_mutex_t setup_m;

	/* both devices are in state AUDIO, creates AmAudio's and runs duplex */
	void connectAudio();
	// hangup pstn
	void hangupPSTN();

	AmRequest* m_req;
	AmCmd m_cmd;
	int m_backendtype;
	int m_pstnstate;
	int m_sipstate;
	bool m_fromPSTN;
	AmSession* m_session;

	char m_fromnumber[MAX_NUM_LEN];
	char m_tonumber[MAX_NUM_LEN];
	char m_fromSip[MAX_URI_LEN];
	char m_toSip[MAX_URI_LEN];

};

#endif


