#ifdef USE_CAPI20

#include "IsdnCapiConnection.h"
#include "IsdnCapiConnectionManager.h"
#include "IsdnCapiUtil.h"
#include "IsdnConfiguration.h"
#include "IsdnUtils.h"
#include "IsdnCallManager.h"
#include "AmApi.h"
#include "globals.h"
#include "log.h"
#include "IsdnErrorCodes.h"
#include <stdio.h>
#include <capi20.h>
#include <pthread.h>
#include <string.h>

IsdnCapiConnection::IsdnCapiConnection(int controller, int t) {
	init();
	m_controller = controller;
	type = t;
}

IsdnCapiConnection::IsdnCapiConnection() {
	init();
	DBG("creating IsdnCapiConnection object, is this ever used?\n");
	type = CC_TYPE_INBOUND;
}

void IsdnCapiConnection::setCall(IsdnCall* call) {
	m_call = call;
}	

IsdnCall* IsdnCapiConnection::getCall() {
	return m_call;
}	

int IsdnCapiConnection::getController() {
	return m_controller;
}	

/* constructor for incoming call */
IsdnCapiConnection* IsdnCapiConnection::incomingCall(_cmsg *msg) {
	IsdnCapiConnection* in_ccon;
	unsigned int cntrl;
	unsigned char from[MAX_NUM_LEN];
	unsigned char to[MAX_NUM_LEN];
	unsigned int fromlen;
	unsigned int tolen;
	int accept;
	int res;
	cntrl = CONNECT_IND_PLCI(msg);
	
	/* complete data struct len of caller */
	fromlen = ((unsigned int) *CONNECT_IND_CALLINGPARTYNUMBER(msg)) - 1; 
	if (fromlen == 1) { /* no number submitted */
		from[0] = '0';
		from[1] = '\0';
	} else if (fromlen > MAX_NUM_LEN) {
		ERROR("caller number too long (%i)\n", fromlen);
		return NULL;
	}	else {
		memcpy(from, CONNECT_IND_CALLINGPARTYNUMBER(msg)+3, fromlen-1);
		*(from+fromlen-1) = '\0';
	}	

	/* complete data struct len of callee */
	if ( ((unsigned int)*CONNECT_IND_CALLEDPARTYNUMBER(msg)) == 0 ) {
		ERROR("callee number data length is zero! Working around this problem by using 0. Check your drivers!\n");
		*to = '0';
		*(to+1) = '\0';
		tolen = 1;
	} else {	
		tolen = ((unsigned int) *CONNECT_IND_CALLEDPARTYNUMBER(msg)) - 1; 
		if (tolen > MAX_NUM_LEN) {
			ERROR("callee number too long (%i)\n", tolen);
			return NULL;
		}	
		memcpy(to, CONNECT_IND_CALLEDPARTYNUMBER(msg)+2, tolen);
		*(to+tolen) = '\0';
	}	

	accept = isdn_config->acceptCallOnDevice(cntrl & 0x0FF, (char*)to);
	DBG("incoming call on controller %i from %s to %s will%s be accepted\n", cntrl & 0x00FF, from, to, accept==OK?"":" not" );
	if (accept == OK) {
		in_ccon = new IsdnCapiConnection(cntrl & 0x00FF, CC_TYPE_INBOUND);
		if (capiManager->registerIncomingCall(cntrl, in_ccon) == FAIL) {
			ERROR("could not register new incoming call\n");
			delete(in_ccon);
			return NULL;
		}
/*		strcpy(in_ccon->m_from, (char*)from);
		strcpy(in_ccon->m_to, (char*)to);
*/
		IsdnCall* call = callManager->placeCallFromPSTN((char*)from, (char*)to, CONNECTIONTYPE_CAPI, in_ccon);
		in_ccon->setCall(call);
		
		return in_ccon;
	} else {
		/* send disconnect_ind to allow subsequent incoming calls... */
		DISCONNECT_REQ_HEADER(msg,capiManager->getAppId(),capiManager->createCCId(),cntrl); 
		res = IsdnCapiUtil::putMessage(msg);
		if (res == FAIL) {
			ERROR("could not put message disconnect_req to indicate denying incoming call.\n");
		}
		DBG("CAPI: rejected call\n");

		return NULL;
	}	
}

void IsdnCapiConnection::init() {
	id = capiManager->createCCId();
	/* init the audio buffers for in&out */
	fi_readpos = fi_audiobuffer;
	fi_datalen = 0;
	ti_readpos = ti_audiobuffer;
	ti_datalen = 0;
	pthread_mutex_init(&fi_m, NULL);
	pthread_mutex_init(&ti_m, NULL);
	//pthread_mutex_init(&callsetup_m, NULL);
	pthread_mutex_init(&adr_m, NULL);
	audioConnected = false;
	cur_databuf = 0;
}

IsdnCapiConnection::~IsdnCapiConnection() {
	capiManager->deleteCall(this);
}

bool IsdnCapiConnection::isIncoming() {
	return (type == CC_TYPE_INBOUND);
}	

int IsdnCapiConnection::placeCall(AmRequest* req, char* phonenumber, char* msn) {
	int shownumber = isdn_config->clip?1:0;
	int ret;
	
	if (strlen(msn) != 0) {
		ret = call(msn, phonenumber, shownumber);
	} else {
		ret = call(NULL, phonenumber, shownumber);
	}
	return ret;
}


void IsdnCapiConnection::onBeforeCallAccept(AmRequest* req) {
	char phonenumber[MAX_NUM_LEN];
	char msn[MAX_NUM_LEN];
	
	if (!isIncoming()) {
		DBG("processing outgoing call\n");
		IsdnUtils::determineNumbers(req, phonenumber, msn, MAX_NUM_LEN); /* throws Exception on error */
		INFO("determine numbers result: from=%s to=%s\n", phonenumber, msn);
		req->reply(183, "isdngw: call progress");
		
		req->reply(180, "isdngw: capi ringing");
	} else {
		DBG("processing incoming call\n");
		m_call->setRequest(req);
		connectIncoming();
	}	
		
}
	
int IsdnCapiConnection::call(char* caller, char* callee, int shownumber) {
	_cmsg msg;
	unsigned char callerstruct[NUM_BUF_LEN];
	unsigned char calleestruct[NUM_BUF_LEN];
	int res;
	int ntype;

	DBG("CAPI call from %s to %s\n", caller, callee);

	CONNECT_REQ_HEADER(&msg, capiManager->getAppId(), 1, m_controller);

	if (caller == NULL) {
		caller = isdn_config->out_msn;
	};

	ntype = shownumber==0?TYPE_CALLER_SUPPRESS:TYPE_CALLER;
	if (IsdnCapiUtil::build_number(ntype, callerstruct, NUM_BUF_LEN, caller) == FAIL) {
		ERROR("error building origin number\n");
		return -1;
	}	

	if (IsdnCapiUtil::build_number(TYPE_CALLEE, calleestruct, NUM_BUF_LEN, callee) == FAIL) {
		ERROR("error building target number\n");
		return -1;
	}	

	//DBG("IsdnCapiConnection::call: from %.*s to %.*s\n", (*calleestruct)-1, calleestruct+2, (*callerstruct)-2, callerstruct+3);

	CONNECT_REQ_CALLEDPARTYNUMBER(&msg)=calleestruct;
	CONNECT_REQ_CALLINGPARTYNUMBER(&msg)=callerstruct;

	CONNECT_REQ_CALLEDPARTYSUBADDRESS(&msg)=0;
	CONNECT_REQ_CALLINGPARTYSUBADDRESS(&msg)=0;
	
	CONNECT_REQ_CIPVALUE(&msg)=16; /* accept speech */

	CONNECT_REQ_B1PROTOCOL(&msg)=1;
	CONNECT_REQ_B2PROTOCOL(&msg)=1;
	CONNECT_REQ_B3PROTOCOL(&msg)=0;
	
	CONNECT_REQ_B1CONFIGURATION(&msg)=NULL;
	CONNECT_REQ_B2CONFIGURATION(&msg)=NULL;
	CONNECT_REQ_B3CONFIGURATION(&msg)=NULL;
	
	msg.Messagenumber = id;
	
	capiManager->registerOutgoingCall(id, this);
	
	res = IsdnCapiUtil::putMessage(&msg);

	if (res == FAIL) {
		ERROR("ERROR: could not put message connect_req\n");
		return FAIL;
	}
		
	return OK;
}

/* connects a pending call, capi still ringing, SIP 200 OK, attach audio */
void IsdnCapiConnection::connectIncoming() { 
	_cmsg msg;
	int res;
	CONNECT_RESP_HEADER(&msg, capiManager->getAppId(), 1, getAdr());
	CONNECT_RESP_B1PROTOCOL(&msg) = 1;
	CONNECT_RESP_B2PROTOCOL(&msg) = 1;
	
	res = IsdnCapiUtil::putMessage(&msg);
	if (res == FAIL) {
		ERROR("could not put message connect_resp\n");
	}
}

int IsdnCapiConnection::cb_connect_conf(_cmsg* msg) {
	DBG("callback funtion called: cb_connect_conf!\n");
	capiManager->updateOutgoingCall(id, msg->adr.adrPLCI, this);
	setAdr(msg->adr.adrPLCI);
	return OK;
}

int IsdnCapiConnection::cb_connect_ind(_cmsg* msg) {
	char fromSip[MAX_URI_LEN];
	char toSip[MAX_URI_LEN];
	char callid[MAX_URI_LEN];
	int res;
	DBG("callback funtion called: cb_connect_ind! (incoming call from >%s< to >%s<)\n", m_call->getFromNumber(), m_call->getToNumber());
	setAdr(CONNECT_IND_PLCI(msg));
	IsdnUtils::buildSipUri(fromSip, MAX_URI_LEN, m_call->getFromNumber(), true);
	IsdnUtils::buildSipUri(toSip, MAX_URI_LEN, m_call->getToNumber(), false);
	strcat(callid, INCOMING_KEY_PREFIX);
	sprintf(callid, "%s%x", INCOMING_KEY_PREFIX, getAdr());
	DBG("using >%s< as identifier for call from >%s< to >%s<\n", callid, fromSip, toSip);
	AmRequestUAC::dialout(callid, MODULE_NAME, toSip, fromSip);

	/* use alert_req to say we are (probably) responsible! */
	ALERT_REQ_HEADER(msg,capiManager->getAppId(),msg->Messagenumber,getAdr());	

	res = IsdnCapiUtil::putMessage(msg);
	if (res == FAIL) {
		ERROR("could not put message alert_req\n");
		return FAIL;
	}

	DBG("set controller %i in listen mode (LISTEN_REQ)\n", (int) (getAdr() & 0xFF)); 
	capiManager->makeControllerListen(getAdr() & 0xFF);
	return OK;
}

int IsdnCapiConnection::cb_alert_conf(_cmsg* msg) {
	DBG("callback funtion called: cb_alert_conf!\n");
	return OK;
}

int IsdnCapiConnection::cb_connect_active_ind(_cmsg* msg) {
	int res;
	_cmsg msg2;
	DBG("callback funtion called: cb_connect_active_ind!\n");

	capi20_cmsg_answer(msg);
	res = IsdnCapiUtil::putMessage(msg);
	if (res == FAIL) {
		ERROR("could not put message connect_active_response\n");
		return FAIL;
	}
	DBG("physical connection established, now creating logical connection\n");
	//CONNECT_B3_REQ_HEADER(&msg2, capiManager->getAppId(), id, msg->adr.adrPLCI);
	CONNECT_B3_REQ_HEADER(&msg2, capiManager->getAppId(), id, getAdr());
	res = IsdnCapiUtil::putMessage(&msg2);
	if (res == FAIL) {
		ERROR("could not put message connect_b3_req\n");
		return FAIL;
	}
	return OK;
}

int IsdnCapiConnection::cb_connect_b3_conf(_cmsg* msg) {
	if (type == CC_TYPE_INBOUND) {
//		capiManager->updateIncomingCall(msg->adr.adrPLCI, this);
	} else {
//		capiManager->updateOutgoingCall(id, msg->adr.adrPLCI, this);
	}
	setAdr(msg->adr.adrPLCI);
	DBG("connected, now unlocking mutex\n");
/* 	m_call->changePSTNState(CALLSTATE_CONNECTED); */
	m_call->unlock();
	return OK;
}

int IsdnCapiConnection::cb_connect_b3_ind(_cmsg* msg) {
	int res;
	setAdr(msg->adr.adrPLCI);
	capi20_cmsg_answer(msg);
	res = IsdnCapiUtil::putMessage(msg);
	if (res == FAIL) {
		ERROR("could not put message connect_b3_resp\n");
		return FAIL;
	}
	return OK;
}

int IsdnCapiConnection::cb_connect_b3_active_ind(_cmsg* msg) {
	int res;
	setAdr(msg->adr.adrPLCI);
	capi20_cmsg_answer(msg);
	res = IsdnCapiUtil::putMessage(msg);
	if (res == FAIL) {
		ERROR("could not put message connect_active_response\n");
		return FAIL;
	}
	return OK;
}

int IsdnCapiConnection::cb_disconnect_ind(_cmsg* msg) {
	int reason;
	int res;
	DBG("IsdnCapiConnection::cb_disconnect_ind\n");
	reason = DISCONNECT_IND_REASON(msg);
	if (reason == 0x349c) {
		ERROR("END CALL: invalid number format\n");
		m_call->changePSTNState(CALLSTATE_DISCONNECTED, 500, "isdngw: invalid number format (capi error: 0x349c)");
	} else if (reason == 0x3491) {
		m_call->changePSTNState(CALLSTATE_DISCONNECTED, 486, "isdngw: busy capi line");
	} else if (reason == 0x3490) { /* normal call clearing */
		m_call->changePSTNState(CALLSTATE_DISCONNECTED);
	} else if (reason == 0x34a2) {
		m_call->changePSTNState(CALLSTATE_DISCONNECTED, 486, "isdngw: busy, no channel/circuit available");
	} else if (reason == 0x3301) {
		m_call->changePSTNState(CALLSTATE_DISCONNECTED, 500, "isdngw: broken line (check cable setup)");
	} else if (reason == 0x3400) { /* normal termination */
		m_call->changePSTNState(CALLSTATE_DISCONNECTED);
	} else if (reason == 0x349a) {
		DBG("another isdn device on the bus took the call, cancelling SIP call\n");
		m_call->changePSTNState(CALLSTATE_DISCONNECTED);
	} else {
		ERROR("unknown capi error, received DISCONNECT_IND with info %x (%s)\n", reason, isdnError(reason));
		m_call->changePSTNState(CALLSTATE_DISCONNECTED, 500, "unknown capi error");
	}	
	capi20_cmsg_answer(msg);
	res = IsdnCapiUtil::putMessage(msg);
	if (res == FAIL) {
		ERROR("could not put message disconnect_resp\n");
		return FAIL;
	}
	capiManager->deleteCall(this);
	
	DBG("cb_disconnect_ind is now unlocking the callsetup mutex, this@%p\n", this);
	m_call->unlock();
	return OK;
}

int IsdnCapiConnection::cb_disconnect_b3_ind(_cmsg* msg) {
	DBG("IsdnCapiConnection::cb_disconnect_b3_ind (media ended)\n");
	m_call->changePSTNState(CALLSTATE_DISCONNECTED);
	capi20_cmsg_answer(msg);
	if (IsdnCapiUtil::putMessage(msg) == FAIL) {
		ERROR("could not put message disconnect_b3_resp\n");
		return FAIL;
	}
	return OK;
}

int IsdnCapiConnection::cb_data_b3_ind(_cmsg* msg) {
	unsigned char* datapointer;
	unsigned int datalength;
	unsigned int res;
	datapointer = DATA_B3_IND_DATA(msg);
	datalength = DATA_B3_IND_DATALENGTH(msg);
	/* reversed-bit alaw -> alaw */
	IsdnCapiUtil::reverseBits(datapointer, datalength);
	/* add to fi buffer */
	audioConnected = true;
	//DBG("received %i bytes\n", datalength);
	pthread_mutex_lock(&fi_m);
	res = writeToBuffer(fi_audiobuffer, fi_readpos, &fi_datalen, datapointer, datalength);
	pthread_mutex_unlock(&fi_m);
	//DBG("done.\n");

	/* response to capi message, buffer may be freed now by capi backend */
	capi20_cmsg_answer(msg);
	res = IsdnCapiUtil::putMessage(msg);
	if (res == FAIL) {
		ERROR("could not put message cb_data_b3_conf\n");
		return FAIL;
	}
	return OK;
}


int IsdnCapiConnection::cb_data_b3_conf(_cmsg* msg) {
	if (DATA_B3_CONF_INFO(msg) == 0x2002)
		ERROR("bug: invalid ncci for DATA_B3_REQ message!\n");
	return OK;
}

int IsdnCapiConnection::cb_disconnect_conf(_cmsg* msg) {
	return OK;
}

int IsdnCapiConnection::accept() {
	return OK;
}

int IsdnCapiConnection::hangup() {
	int res;
	_cmsg msg;
	DISCONNECT_REQ_HEADER(&msg,capiManager->getAppId(),capiManager->createCCId(),getAdr()); 
	res = IsdnCapiUtil::putMessage(&msg);
	if (m_call != NULL)
		m_call->unlock(); /* make shure the call thread does not hang */
	
	if (res == FAIL) {
		ERROR("could not put message disconnect_req\n");
		return FAIL;
	}
	//DBG("############################################# just sent disconnect\n");
	return OK;
}

/* returns audio data from the isdn connection to sems 
 * no data: 0
 * never more data: -1 
 * other: bytes transfered
 */
int IsdnCapiConnection::getVoiceStream(unsigned char *buf, int size) {
	int res;
/*	if (state != CC_STATE_AUDIO) {
		if ((state == CC_STATE_WASTE) || (state == CC_STATE_DISCONNECTED)) {
			return -1;
		}	
		ERROR("getVoiceStream for CAPI device called, but state != AUDIO\n");
		return 0;
	}
*/	
	pthread_mutex_lock(&fi_m);
/* need somehow a chunk size? -> uncomment */
	/*
	if ((fi_datalen < (unsigned)size) && (!m_call->isDisconnected())) {
		DBG("not enough data avail (%i/%i). proceed next time\n", fi_datalen, size);
		pthread_mutex_unlock(&fi_m);
		return 0;
	}	
	*/
	res = readFromBuffer(fi_audiobuffer, &fi_readpos, &fi_datalen, buf, size);
	pthread_mutex_unlock(&fi_m);
/*	DBG("returned %i bytes of audio from capi\n", res);*/
	if (res%2!=0) res--;
	return res;
}	

int IsdnCapiConnection::putVoiceStream(unsigned char *buf, int size) {
	unsigned int res;
	unsigned char buffer[MSG_SIZE];
	unsigned char* readpos_save;
	unsigned int datalen_save;
	_cmsg msg;
	int ti_bytes;
/*	if (state != CC_STATE_AUDIO) {
		if ((state == CC_STATE_WASTE) || (state == CC_STATE_DISCONNECTED )) {
			return -1;
		}	
		ERROR("putVoiceStream for CAPI device called, but state != AUDIO, dropping %i bytes\n", size);
		return 0;
		
	}	
*/	
	if (m_call->isDisconnected())
		return -1;

	if (!audioConnected) {
		DBG("putVoiceStream: !audioConnected, dropping data\n");
		return size;
	}	
		
	IsdnCapiUtil::reverseBits(buf, size); /* drop as reversed alaw in memory */
	pthread_mutex_lock(&ti_m);
	res = writeToBuffer(ti_audiobuffer, ti_readpos, &ti_datalen, buf, size);
	//DBG("just wrote %i bytes to ti buffer\n", res); 
	readpos_save = ti_readpos;
	datalen_save = ti_datalen;
	
	ti_bytes = ti_datalen < MSG_SIZE ? ti_datalen : MSG_SIZE;
	if (ti_bytes < AUDIO_CHUNK_SIZE) {
		//DBG("no write, only %i bytes available\n", ti_bytes); 
		goto skiptiwrite;
	}	
	

	if (cur_databuf++ == MAX_DATABUF-1)
		cur_databuf=0;
		
	res = readFromBuffer(ti_audiobuffer, &ti_readpos, &ti_datalen, audiotosend[cur_databuf], ti_bytes);
	
	DATA_B3_REQ_HEADER(&msg,capiManager->getAppId(),id,getAdr());
	DATA_B3_REQ_DATA(&msg) = audiotosend[cur_databuf];
	DATA_B3_REQ_DATALENGTH(&msg) = ti_bytes;
	DATA_B3_REQ_DATAHANDLE(&msg) = cur_databuf;
	capi_cmsg2message(&msg, buffer);
	res = CAPI20_PUT_MESSAGE(capiManager->getAppId(), buffer);
	if (res != CapiNoError) { // could not send data, rollback 
//		ti_readpos = readpos_save;
//		ti_datalen = datalen_save;
		ERROR("CAPI audio buffer full, dropping data.\n");
	}	
	
	//DBG("wrote %i bytes (available: %i bytes) with handle %i\n", ti_bytes, temp, cur_databuf);
	skiptiwrite:
	pthread_mutex_unlock(&ti_m);
	/* TODO: directly write data to the isdn device ! */
	
	return res;
}

int IsdnCapiConnection::writeToBuffer(unsigned char* audiobuffer, unsigned char* readpos, unsigned int* datalen, unsigned char* data, unsigned int bytes) {
	unsigned int res;
	unsigned int writeoffset;
	unsigned int bisEnde;
	if (bytes > AUDIO_BUFFER_LEN - *datalen) {
		res = AUDIO_BUFFER_LEN - *datalen; /* drop data */
		ERROR("WARNING: dropping data, try increase AUDIO_BUFFER_LEN...\n");
	} else {
		res = bytes;
	}

	writeoffset = (readpos - audiobuffer) + *datalen;
	if (writeoffset >= AUDIO_BUFFER_LEN) {
		writeoffset -= AUDIO_BUFFER_LEN;
		memcpy(audiobuffer + writeoffset, data, res);
		/*done*/
	}	else {
		bisEnde = AUDIO_BUFFER_LEN - writeoffset;
		if (res <= bisEnde) {
			memcpy(audiobuffer + writeoffset, data, res);
			/*done*/
		} else {
			memcpy(audiobuffer + writeoffset, data, bisEnde);
			memcpy(audiobuffer, data+bisEnde, res - bisEnde);
			/*done*/
		}
	}

	*datalen += res;
	return res;
}	

int IsdnCapiConnection::readFromBuffer(unsigned char* audiobuffer, unsigned char** readpos, unsigned int *datalen, unsigned char* targetbuffer, int bytes) {
	int result;
	int bisEnde;
	if ((unsigned)bytes > *datalen) 
		result = *datalen;
	else 
		result = bytes;
	
	if (result == 0)
		return result;
	bisEnde = AUDIO_BUFFER_LEN - ( *readpos - audiobuffer );
	if (bisEnde >= result) { /* enough data in one block */
		memcpy(targetbuffer, *readpos, result);
	} else {
		/* copy first block */
		memcpy(targetbuffer, *readpos, bisEnde);
		/* copy rest from start of buffer */
		memcpy(targetbuffer+bisEnde, audiobuffer, result - bisEnde);
	}
	*readpos += result;
	if (*readpos - audiobuffer > AUDIO_BUFFER_LEN)
		*readpos -= AUDIO_BUFFER_LEN;
	*datalen -= result;
	return result;
}

void IsdnCapiConnection::lockAdr() {
	pthread_mutex_lock(&adr_m);
}

void IsdnCapiConnection::unlockAdr() {
	pthread_mutex_unlock(&adr_m);
}

void IsdnCapiConnection::setAdr(unsigned int x) {
	lockAdr();
	adr = x;
	unlockAdr();
}

unsigned int IsdnCapiConnection::getAdr() {
	unsigned x;
	lockAdr();
	x = adr;
	unlockAdr();
	return x;
}

bool IsdnCapiConnection::getAudioConnected() {
	return audioConnected;
}

#endif /*USE_CAPI20*/
