#ifdef USE_CAPI20

#include "IsdnCapiConnectionManager.h"
#include "IsdnCapiUtil.h"
#include "IsdnConfiguration.h"
#include "log.h"
#include <stdio.h>
#include <capi20.h>
#include <linux/capi.h>
#include <pthread.h>
#ifdef USE_TESTCAPI
#include "IsdnCapiTest.h"
#endif

IsdnCapiConnectionManager::IsdnCapiConnectionManager() {
}	

IsdnCapiConnectionManager::~IsdnCapiConnectionManager() {
	DBG("relase CAPI...\n");
	capi20_release(appId);
}	

void* _thread_start(void* cm) {
	IsdnCapiConnectionManager *cur = (IsdnCapiConnectionManager *)cm;
	cur->messageloop();
	return NULL;
}	

int IsdnCapiConnectionManager::init() {
	int res;
	int i;
	int cnum;
	unsigned char kilobuffer[1000];
	unsigned char man_buf[1000];
	struct capi_profile capiprofile;
	pthread_t loop_t;
	
	pthread_mutex_init(&capi_m, NULL);
	/* connection ids start at 5 */
	ccids = 5;

	/* reset avail states */
	for (i = 1; i<=MAX_CAPI_CONTROLLERS; i++)
		isdn_config->devices[i].available = FAIL;

	res = capi20_isinstalled();
	if (res != 0) {
		ERROR("no CAPI backend found on your system, please make shure all neccessary drivers are loadad.\n");
		return FAIL;
	}


	res = capi20_register(5, 5, MSG_SIZE, &appId);
	if (res != 0) {
		ERROR("ERROR: could not register CAPI application, reason %x\n", res);
		return FAIL;
	}	
	DBG("CAPI 2.0 backend found on this system. isdngw registered as application #%i\n", appId);

	res = capi20_get_profile(0, (unsigned char*)&capiprofile);
	cnum = capiprofile.ncontroller;
	DBG("%i CAPI controller%s installed:\n", cnum, cnum==1?"":"s");

	for (i=1; i<=cnum; i++) {
		capi20_get_manufacturer(i, man_buf);
		capi20_get_version(i, kilobuffer);
		res = capi20_get_profile(i, (unsigned char*)&capiprofile);
		if (res != CapiNoError) {
			ERROR("Controller %i not available!\n", i);
			isdn_config->devices[i].numberOfBChannels = -1;
			isdn_config->devices[i].availBChannels = -1;
			isdn_config->devices[i].available = FAIL;
			continue;
		}
		res = capiprofile.nbchannel;
		DBG("   controller %i up (%s, %i B-channels available)\n", i, man_buf, res);
		sprintf(isdn_config->devices[i].identification, "%s", man_buf);
		isdn_config->devices[i].numberOfBChannels = res;
		if (isdn_config->devices[i].availBChannels == -1)
			isdn_config->devices[i].availBChannels = res;
		isdn_config->devices[i].available = OK;
		
		makeControllerListen(i);
		
	}


	/* initialize the ccm lock */
	pthread_mutex_init(&ccm_m, NULL);

	/* initialize the tables */
	for (int slot=0; slot < MAX_CONN; slot++) {
		msg_table[slot] = -1;
		adr_table[slot] = 0;
		cbob_table[slot] = NULL;
	}

	/* starting the message processor loop as a seperate thread */
	pthread_create(&loop_t, NULL, _thread_start, this);
	pthread_detach(loop_t);
	pthread_create(&loop_t, NULL, _thread_start, this);
	pthread_detach(loop_t);

	return OK;
}	


void IsdnCapiConnectionManager::makeControllerListen(int device) {
	_cmsg msg;

	LISTEN_REQ_HEADER(&msg, appId, 1, device);
	LISTEN_REQ_CIPMASK(&msg) = 0x00010012;
	LISTEN_REQ_INFOMASK(&msg) = 0x0;

	LISTEN_REQ_CALLINGPARTYNUMBER(&msg) = 0;
	LISTEN_REQ_CALLINGPARTYSUBADDRESS(&msg) = 0;

	if (IsdnCapiUtil::putMessage(&msg, appId) == FAIL) {
		ERROR("could not put message LISTEN_REQ for controller %i, *not* listening on this device.\n", device);
		return;
	}	
}

#define processMsg(CURMSG,curmsg)	/*DBG(CURMSG);*/ \
			cc_obj = getObjectForMessage(&msg); \
			if (cc_obj == NULL) { \
				DBG("DEBUG: could not find object for callback ("CURMSG")\n"); \
				DBG("DEBUG: this could be a bug!\n"); \
				continue; \
			} \
			if ((cc_obj->curmsg)(&msg) == FAIL) { \
				ERROR("callback failed, trying to live on\n"); \
				return; \
			}	

void IsdnCapiConnectionManager::messageloop() {
	struct timeval tv;
	int res;
	_cmsg msg;
	IsdnCapiConnection* cc_obj;
	#ifdef USE_TESTCAPI
	IsdnCapiTestReader testreader = IsdnCapiTestReader("testreader.al");
	#endif

	DBG("starting CAPI message processor loop...\n");

	while (1) {
		#ifdef USE_TESTCAPI
		testreader.action();
		#endif
		tv.tv_sec  = 10L; /* 10 second timeout */
		tv.tv_usec = 0L;
		res = CAPI20_WaitforMessage(appId, &tv);
		if (res == 4356) {
			//DBG("timeout\n"); 
			continue;
		}	else if (res == 4353) {
			ERROR("illegal application identifier, internal error, exit\n");
			return;
		}	else if (res == 4354) {
			ERROR("Illegal command or subcommand or message length less than 12 bytes, internal error, try to live on!\n");
			continue;
		}	else if (res != CapiNoError) {
			ERROR("an unknwon error occurred (%i) while waitForMessage, trying to live on\n", res);
			continue;
		}	
		
		lockCAPI();
		res = capi_get_cmsg(&msg, appId);
		unlockCAPI();
		if (res == 4356) {
				// DBG("another thread took the message\n");
				continue; /* another loop thread took the message */
			}
		if (res != CapiNoError) {
			ERROR("could not receive message, reason %s (%i)\n", capi_info2str(res), res);
			continue;
		}	
		if (IS_CONNECT_IND(&msg)) {
			/* check for incoming call */
			cc_obj = getObjectForMessage(&msg); 
			if (cc_obj == NULL) { /* new incoming call! */
				DBG("incoming call deteced\n");
				IsdnCapiConnection* in_ccon;
				in_ccon = IsdnCapiConnection::incomingCall(&msg);
				if (in_ccon != NULL) {
					DBG("incoming call ok, processing message\n");
					processMsg("CONNECT_IND",cb_connect_ind);
				}	/* if NULL, call not handled by isdngw */
			} else {
				processMsg("CONNECT_IND",cb_connect_ind);
			}
		} else if (IS_ALERT_CONF(&msg)) {
			processMsg("ALERT_CONF",cb_alert_conf);
		} else if (IS_CONNECT_CONF(&msg)) {
			processMsg("CONNECT_CONF",cb_connect_conf);
		} else if (IS_DISCONNECT_IND(&msg)) {
			processMsg("DISCONNECT_IND",cb_disconnect_ind);
		} else if (IS_DISCONNECT_CONF(&msg)) {
			/*processMsg("DISCONNECT_CONF",cb_disconnect_conf);*/ /* noop'ed (not needed) */
		} else if (IS_DISCONNECT_B3_IND(&msg)) {
			processMsg("DISCONNECT_B3_IND",cb_disconnect_b3_ind);
		} else if (IS_CONNECT_ACTIVE_IND(&msg)) {
			processMsg("CONNECT_ACTIVE_IND",cb_connect_active_ind);
		} else if (IS_CONNECT_B3_IND(&msg)) {
			processMsg("CONNECT_B3_IND",cb_connect_b3_ind);
		} else if (IS_CONNECT_B3_CONF(&msg)) {
			processMsg("CONNECT_B3_CONF",cb_connect_b3_conf);
		} else if (IS_CONNECT_B3_ACTIVE_IND(&msg)) {
			processMsg("CONNECT_B3_ACTIVE_IND",cb_connect_b3_active_ind);
		} else if (IS_LISTEN_CONF(&msg)) {
			/*processMsg("LISTEN_CONF",cb_listen_conf);*/
			/*DBG("LISTEN_CONF received\n");*/
		} else if (IS_DATA_B3_IND(&msg)) {
			processMsg("DATA_B3_IND",cb_data_b3_ind);
			#ifdef USE_TESTCAPI
			testreader.setConnection(cc_obj);
			#endif
		} else if (IS_DATA_B3_CONF(&msg)) {
			processMsg("DATA_B3_CONF",cb_data_b3_conf);
		} else {
			ERROR("CMD=%x, SUBCMD=%x -> ", msg.Command, msg.Subcommand); 
			ERROR("unknown message received\n");
			ERROR("************************** PLEASE REPORT A BUG TO sems@iptel.org *************\n");
			continue;
		}	
	}
}	

unsigned int IsdnCapiConnectionManager::getAppId() {
	return appId;
}	

int IsdnCapiConnectionManager::processMessage(_cmsg *msg) {


	return OK;
}

unsigned int IsdnCapiConnectionManager::createCCId() {
	int ret;
	lock();
	ret = ccids++;
	if (ccids > 10000)
		ccids = 4;
	unlock();
	return ret;
}	

#define _plci(x) ((x & 0x0000FF00) >> 8)
#define _controller(x) (x & 0x000000FF)

#define debugAdr(x) DBG("ADR: [%lx] [%lx] [%lx] [%lx]\n", (x & 0xFF000000) >> 24, (x & 0x00FF0000) >> 16, _plci(x), _controller(x));

#define dumpTable for(int i=0; i<MAX_CONN; i++) { DBG("slot #%i msgid=%i ADR: [%lx] [%lx] [%lx] [%lx] OBJ@%p\n", i, msg_table[i], (adr_table[i] & 0xFF000000) >> 24, (adr_table[i] & 0x00FF0000) >> 16, _plci(adr_table[i]), _controller(adr_table[i]), cbob_table[i]); }

unsigned int IsdnCapiConnectionManager::registerOutgoingCall(unsigned int msgid, IsdnCapiConnection* cb_obj) {
	int slot;
	
	DBG("registerOutgoingCall: msgid=%i ccon@%p\n", msgid, cb_obj);
	lock();
	if (getSlotForMessageId(msgid) != NO_SLOT_FOUND) {
		ERROR("table already contains an entry for msgid=%i\n", msgid);
	}	
	
	slot = getFreeSlot();
	if (slot == NO_SLOT_FOUND) {
		ERROR("ERROR: no free slot available to register another outgoing call.\n");
		unlock();
		return FAIL;
	}
	msg_table[slot] = msgid;
	cbob_table[slot] = cb_obj;
	unlock();
	return OK;
}	

unsigned int IsdnCapiConnectionManager::updateOutgoingCall(unsigned int msgid, unsigned long adr, IsdnCapiConnection* cb_obj) {
	int slot;
	int res;
	DBG("updateOutgoingCall: msgid=%i ADR:\n", msgid);
	debugAdr(adr);
	lock();
	slot = getSlotForMessageId(msgid);
	if (slot != NO_SLOT_FOUND) {
		cbob_table[slot] = cb_obj;
		adr_table[slot] = adr;
		res = OK;
	}	else {
		ERROR("could not find slot to update (msgid=%i)\n", msgid);
		res = FAIL;
	}
	unlock();
	return res;
}

unsigned int IsdnCapiConnectionManager::registerIncomingCall(unsigned long adr, IsdnCapiConnection* cb_obj) {
	int slot;
	DBG("registerIncomingCall: ADR: \n");
	debugAdr(adr);
	lock();
	slot = getFreeSlot();
	if (slot == NO_SLOT_FOUND) {
		ERROR("ERROR: no free slot available to register another incoming call.\n");
		unlock();
		return FAIL;
	}
	msg_table[slot] = 1;
	cbob_table[slot] = cb_obj;
	adr_table[slot] = adr;
	unlock();
	
	return OK;
}

void IsdnCapiConnectionManager::deleteCall(IsdnCapiConnection* cb_obj) {
	//DBG("deleteCall: obj@%p\n", cb_obj);
	for (int i=0; i<MAX_CONN; i++) 
		if (cbob_table[i] == cb_obj) {
			msg_table[i] = -1;
			adr_table[i] = 0l;
			cbob_table[i] = NULL;
		}	
}

int IsdnCapiConnectionManager::getFreeSlot() { /* not sync'ed! do outside! */	
	int slot;
	for (slot=0; slot < MAX_CONN; slot++)
		if (msg_table[slot] == -1)
			/* found a free slot */
			return slot;
	return NO_SLOT_FOUND;
}

int IsdnCapiConnectionManager::getSlotForMessageId(unsigned int msgid) { /* not sync'ed */
	int slot;
	for (slot=0; slot < MAX_CONN; slot++)
		if ((msg_table[slot]>=0) && (((unsigned)msg_table[slot]) == msgid))
			return slot;
	return NO_SLOT_FOUND;		
}

int IsdnCapiConnectionManager::getSlotForAdr(unsigned long adr) { /* not sync'ed */
	int slot;
	for (slot=0; slot < MAX_CONN; slot++) {
		if ((_plci(adr_table[slot]) == _plci(adr)) && (_controller(adr_table[slot]) == _controller(adr))) {
			return slot;
		}		
	}
	return NO_SLOT_FOUND;		
}

IsdnCapiConnection* IsdnCapiConnectionManager::getObjectForMessage(_cmsg *msg) {
	unsigned long l_adr;
	unsigned int l_msgid;
	int res;

	l_adr = msg->adr.adrPLCI;
	l_msgid = msg->Messagenumber;

//	DBG("getObjectForMessage: ADR follows\n");
//  debugAdr(l_adr);

	res = getSlotForMessageId(l_msgid);
	if (res != NO_SLOT_FOUND) {
//		DBG("found slot for messageid %i at %i, object at %p\n", l_msgid, res, cbob_table[res]);
		return cbob_table[res];
	}	

	res = getSlotForAdr(l_adr);
	if (res != NO_SLOT_FOUND) {
//		DBG("found slot for message at %i, object at %p\n", res, cbob_table[res]);
		return cbob_table[res];
	}	
		
	DBG("getObjectForMessage: Could not find object for msg (msgid=%i, adr follows) - incoming?\n", l_msgid);
	debugAdr(l_adr);
	return FAIL;
}

IsdnCapiConnection* IsdnCapiConnectionManager::getPendingCallConnection(const char* callid) {
	unsigned n;
	int sl;

	if (sscanf(callid, INCOMING_KEY_PREFIX"%x",&n) == 1) {
		sl = getSlotForAdr(n);
		if (sl != NO_SLOT_FOUND) {
			/* ok */
			DBG("found matching capi connection at slot %i\n", sl);
			if (cbob_table[sl] == NULL)
				ERROR("internal error, slot %i found, but object is NULL\n", sl);
			return cbob_table[sl];
		}	
	}
	DBG("no capi connection matches!\n");
	return NULL;
}

void IsdnCapiConnectionManager::lock() {
	pthread_mutex_lock(&ccm_m);
}

void IsdnCapiConnectionManager::lockCAPI() {
	pthread_mutex_lock(&capi_m);
}

void IsdnCapiConnectionManager::unlock() {
	pthread_mutex_unlock(&ccm_m);
}

void IsdnCapiConnectionManager::unlockCAPI() {
	pthread_mutex_unlock(&capi_m);
}

#endif 
