#ifdef USE_CAPI20

#ifndef _ISDNCAPICONNECTIONMANAGER_H_
#define _ISDNCAPICONNECTIONMANAGER_H_

#include <capi20.h>
#include <pthread.h>
#include "IsdnCapiUtil.h"
#include "IsdnCapiConnection.h"

#define NO_SLOT_FOUND -1
#define DEFAULT_CALL_TIMEOUT 60

class IsdnCapiConnectionManager {
	
	public:
	IsdnCapiConnectionManager();
	~IsdnCapiConnectionManager();
	
	/* initialize CAPI interface */
 	int init();

	/* neverending loop processing capi messages */
	void messageloop();

	/* returns the appId */
	unsigned int getAppId();

	/* returns an unique id used as message number for call setups */
	unsigned int createCCId();

	unsigned int registerOutgoingCall(unsigned int msgid, IsdnCapiConnection* cb_obj);
	unsigned int updateOutgoingCall(unsigned int msgid, unsigned long adr, IsdnCapiConnection* cb_obj);
	
	unsigned int registerIncomingCall(unsigned long adr, IsdnCapiConnection* cb_obj);

	void deleteCall(IsdnCapiConnection* cb_obj);

	IsdnCapiConnection* getPendingCallConnection(const char* callid);

	void makeControllerListen(int device);

	void lockCAPI();
	void unlockCAPI();

	private:

	/* process a CAPI message from the kernel */
	int processMessage(_cmsg *msg);

	/* CAPI application identifier, needed for all CAPI messages */
	unsigned int appId; 

	/* sysnchronization for capiManager internal structures */
	void lock();
	void unlock();

	unsigned int registerCall(unsigned int ccid, unsigned long ncci, IsdnCapiConnection* cb_obj);
	int getFreeSlot();
	int getSlotForMessageId(unsigned int msgid);
	int getSlotForAdr(unsigned long ncci);
	IsdnCapiConnection* getObjectForMessage(_cmsg *msg);

	unsigned int ccids;
	
	int msg_table[MAX_CONN]; /* stores per-slot message ids */
	unsigned long adr_table[MAX_CONN]; /* stores per-slot adr records */
	IsdnCapiConnection* cbob_table[MAX_CONN]; /* stores corresponding ICC objects */

	pthread_mutex_t ccm_m;
	pthread_mutex_t capi_m;
};

extern IsdnCapiConnectionManager *capiManager;

#endif /* header */
#endif /* USE_CAPI20 */
