#ifdef USE_CAPI20

#include "IsdnCapiUtil.h"
#include "IsdnCapiConnectionManager.h"
#include "log.h"
#include <string.h>
#include <stdio.h>

/* creates a CAPI compliant number struct */
/*static*/ int IsdnCapiUtil::build_number(int type, unsigned char* buf, int len, const char* number) {
	int i=0;
	int numberlen;

	/* check if buf is large enough */
	numberlen = strlen(number);
	if (numberlen > len+3) {
		ERROR("error, buffer to small to build number\n");
		return FAIL;
	}

	if (type == TYPE_CALLEE) {
		*(buf++)=1+numberlen;
		*(buf++)=0x80;
	} else if (type == TYPE_CALLER) {
		*(buf++)=2+numberlen;
		*(buf++)=0x00;
		*(buf++)=0x80;
	} else if (type == TYPE_CALLER_SUPPRESS) {
		*(buf++)=2+numberlen;
		*(buf++)=0x00;
		*(buf++)=0xA0;
	} else {
		ERROR("invalid type %i\n", type);
		return FAIL;
	}	

	while (i<numberlen) {
		*(buf++) = *(number+i);
		i++;
	}	

	return OK;
}

/* used for bit debugging */
/*static*/ void IsdnCapiUtil::getBits(unsigned char data, char* buf, int reversed) {
	char* mybit = buf;
	unsigned char bit = 1;
	int bitcount = 0;
	mybit[7] = '\0';
	
	if (reversed == 0) {
		while (bit) {
			if (data & bit) 
				mybit[7-bitcount] = '1';
			else 
				mybit[7-bitcount] = '0';
			bitcount++;
			bit = bit << 1;
		}	
	} else {
		while (bit) {
			if (data & bit)
				mybit[bitcount] = '1';
			else
				mybit[bitcount] = '0';
			bitcount++;
			bit = bit << 1;
		}	
	}	
}

int IsdnCapiUtil::putMessage(_cmsg *msg, unsigned int appId) {
	unsigned char buf[MSG_SIZE];
	int res;
	if (appId == 0)
		appId = capiManager->getAppId();
	capi_cmsg2message(msg, buf);
	res = CAPI20_PUT_MESSAGE(appId, buf);
	if (res != CapiNoError) {
		ERROR("ERROR: could not put CAPI message, reason: %s (%i)\n", capi20_info2str(res), res);
		return FAIL;
	}	
	return OK;
}

/*static*/ void IsdnCapiUtil::reverseBits(unsigned char* start, unsigned int len) {
	while (len-- > 0)
		reverseBits(start++);
}

/* 0100 1000 -> 0001 0010, needed to convert reversed-bit aLaw to normal aLaw */
/*static inline*/ void IsdnCapiUtil::reverseBits(unsigned char *data) {
	unsigned char bit = 1;
	unsigned char reversed = 0;
	int bitcount = 0;

	while (bit) {
		if (*data & bit)
			reversed |= 128 >> bitcount;
		bitcount++;
		bit = bit << 1;
	}	
	*data = reversed;
}

	
#endif
