/*
 * $Id: IsdnChat.h,v 1.1 2003/08/28 16:19:58 ullstar Exp $
 *
 * Copyright (C) 2003 Ulrich Abend
 *
 * This file is part of isdngw, a free isdn gateway plugin 
 * for sems, a free SIP media server.
 *
 * isdngw is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * For a license to use the ser software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * isdngw is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _ISDNCHAT_H_
#define _ISDNCHAT_H_

/** /dev/ttyI* device dialog implementation. Instances of this class 
 * are a description of a "chat" with isdn devices.
 * The bahavior can be defined by use of one of the setter methods maybe
 * in combination with addErrorHandler() and/or addChatSequence(). Once it is
 * completely desribed, chat() can be used to actually perform the "chat".
 * The method setTimeout() allows to set a custom timeout with an associated 
 * return code, if not used a default timeout is used.
 */
class IsdnChat {

	private:
	void initialize ();
//	void stripresult (char* buf); moved to IsdnUtils
	int complete_response (char* buf);
	int check (char* response);

	/* struct to store all chat steps in a list */
	typedef struct chatSequence_t {
		char* command;
		char* response; 
		char options;
		chatSequence_t *next;
	};

	/* struct to store all custom error handlers */
	typedef struct errorList_t {
		char* response; 
		int returncode;
		errorList_t *next;
	};

	/* custom timeout member */
	int custom_timeout;
	/* code to be returned on custom timeout */
	int custom_timeout_returncode;

	/* list start structs for chatsequences and error handlers */
	chatSequence_t *chatListStart;
	errorList_t *errorListStart;

	void freeList(chatSequence_t *cur);
	void freeList(errorList_t *cur);
	void setMsn(char *msn, bool listen = false);
	
	public:
	IsdnChat();
	~IsdnChat();
	
	/* emulates delete and new */
	void reset();
		
	void setInitialization();
	void setOutgoingCall(char* msn, char* number);
	void setIncomingDevice(char* msn);
	void setWaitForAnswer();
	void setConnect();
	void setAnswerIncomingCall();
	void setHangup();
	
	void addChatSequence(char* command, char* response, char options = NO_OPTIONS);
	void addErrorHandler(char* response, int returncode);
	void setTimeout(int tenthsecs, int returncode);
	void debugList();

	int chat (int fd);
	
};

#endif 
