/*
 * $Id: IsdnConfiguration.h,v 1.7 2004/04/04 14:11:28 ullstar Exp $
 *
 * Copyright (C) 2003 Ulrich Abend
 *
 * This file is part of isdngw, a free isdn gateway plugin 
 * for sems, a free SIP media server.
 *
 * isdngw is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * For a license to use the ser software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * isdngw is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _ISDNCONFIGURATION_H_
#define _ISDNCONFIGURATION_H_

#include "SemsConfiguration.h"
#include "IsdnGatewayFactory.h"
#include "AmApi.h"
#include "globals.h"
#include <stdio.h>   /* for FILE in get_line */
#include <string>

/** not initialized yet */
#define STATUS_UNINITIALIZED 0
/** device in valid state */
#define STATUS_OK 1
/** device in invalid state */
#define STATUS_INVALID 2


/* linked list structure */
struct list_t {  
	char* item;
	void* link; /* link to other information */
	int num; /* multi purpose int var */
	struct list_t* next;
};


/** Configuration class for the isdn gateway. Loads, parses and checks 
 * configuration file.
 */
class IsdnConfiguration
 : SemsConfiguration
{
	private:
  IsdnConfiguration();
	int firstfreedevice;
	char errorAsString[100];

	bool _createList(struct list_t*, char*);
	struct list_t* createList(char* list);
	struct list_t* createI4lDeviceList(int num, int start);

	void freeList(struct list_t* list);

	/* device handling and selecting support */
	int _setDeviceMapping(int device, char* list, bool inbound);
	int setDeviceOutboundMapping(int device, char* list);
	int setDeviceInboundMapping(int device, char* list);
	int setNumberOfBChannels(int device, char* number);

	void addSorted(struct list_t* target, struct list_t* cur, int device);
	void addNumberPatterns(int device, struct list_t* list, bool inbound);

	struct deviceconfig_t {
		struct list_t* inboundPatterns;
		struct list_t* outboundPatterns;
		char identification[100];
		int numberOfBChannels;
		int available; /* OK or FAIL */
		int availBChannels; /* -1 for unknown */
	};

	struct list_t* sortedInboundPatterns;
	struct list_t* sortedOutboundPatterns;

	void remove_lockfiles(struct list_t* i4l_devicelist);
	int getLongestMatch(struct list_t* list, char* number);
	

	public:
  virtual ~IsdnConfiguration();
	static void instantiate();
	void reload();

	bool enablei4l;        /* enable/disable these backends */
	bool enablecapi;

	char* out_msn;         /* outgoing connections' msn */
	char* out_logfile;     /* logfile for outgoing calls */
	char* lockdir;         /* directory where lock files are stored */
	char* callerdomain;    /* incoming calls have this caller domain */
	char* calleedomain;    /* incoming calls have this callee domain */
	unsigned minnumberlen; /* outgoing calls have numbers of this len at least */
	bool msnsipcaller;     /* outgoing calls have the callers number as msn */
	bool clip;             /* show number */
	
	struct list_t* i4l_outdevices;
	struct list_t* i4l_indevices;
	
	/* struct storing all device information 0=i4l, 1=capi1, 2=capi2.. */
	struct deviceconfig_t devices[MAX_CAPI_CONTROLLERS+1];

	int acceptCallOnDevice(int device, char* number);
	int getDeviceForCall(char* tonumber, char* fromnumber = NULL);
	void freeLineAfterCall(int devicenumber);
	int check();

	void getListAsString (list_t* list, char* buf, int len);
	void dumpConfiguredDevices ();
	void fifo_dumpdevices (FILE*);
	void debugDumpList (struct list_t* list);
	void debugDumpDeviceList ();

};

/** Globally available IsdnConfiguration instance, should be used
 * to access configuation values.
 */
extern IsdnConfiguration* isdn_config;

#endif
