/*
 * $Id: IsdnDevice.h,v 1.3 2003/12/04 10:06:21 ullstar Exp $
 *
 * Copyright (C) 2003 Ulrich Abend
 *
 * This file is part of isdngw, a free isdn gateway plugin 
 * for sems, a free SIP media server.
 *
 * isdngw is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * For a license to use the ser software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * isdngw is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef _ISDNDEVICE_H_
#define _ISDNDEVICE_H_

#include <pthread.h>   /* for mutex var */
#include "globals.h"   /* for FAIL */
#include "AmRequest.h"   /* for onBeforeCallAccept's request */
#include "IsdnCall.h" 

#define STATUS_UNINIT 0x00
#define STATUS_LOCKED 0x01
#define STATUS_OPEN 	0x02

/** An isdn device. Takes care of opening closing, locking, lock file 
 * generation of isdn devices.
 * High level methods are provided for common actions like dial() or hangup(),
 * which make use of IsdnChat. Methods specific to in- and outgoing connections
 * have moved to the IsdnDeviceIncoming and IsdnDeviceOutgoing classes.
 */
class IsdnDevice
{
private:
	char* m_lockfilename;    /* name of the lockfile */
	pthread_mutex_t mut_dev; /* mutex to protect the device */
	int createLockFile();    /* try to create a lock file */
	int m_status;            /* lock file and fd status */
	int open_device();       /* open the device in nonblocking mode */
	int setTerminalParameters();  /* use termios to set parameters */
	int filterSpecials(unsigned char *buf, int *size);  /* filter for 0x10 */
	void flush();            /* flush input and output buffer */
protected:
	IsdnCall* m_call;
	char* m_devicename;      /* /dev/ttyI* */
	int m_fd;                /* file descriptor */
	char* m_incoming_number; /* number to listen on (msn) */
	pthread_mutex_t mut_hup; /* mutex to protect call clearing */
	int m_request_hangup;    /* store if hangup is requested */
	//pthread_mutex_t m_call;  /* mutex to protect call accept on pstn side */
	//pthread_t t_listener;    /* listening thread for this isdn device */
	
public:
	IsdnDevice(char* devicename);
	virtual ~IsdnDevice();

	void setCall(IsdnCall*);
	IsdnCall* getCall();

	virtual void onBeforeCallAccept(AmRequest* req)=0;
	virtual void onError(unsigned int code, const string& reason)=0;
	virtual void onBye()=0;
	
	virtual int connectVoice()=0;
	
	virtual void cleanup() {};
	int trylock();
	// static IsdnDevice* getLockedDevice(int inout);
	int initialize();
	
	/* hangup a pending call */
	void hangup();

	void unlock();     /* unlock the device, must be reinitialized */

	/* these are used for actual voice transfer */
	int getVoiceStream(unsigned char *buf, int size);  /* get audio bytes */
	int putVoiceStream(unsigned char *buf, int size);  /* put audio bytes */

	/* returns "INCOMING" or "OUTGOING" (for debugging only) */
	virtual const char* getType()=0;
	
	void resetHangupRequested();
	void setHangupRequested();
	int getHangupRequested();
	int removeLockFile();    /* remove the lockfile */
};

#endif
