/*
 * $Id: IsdnDeviceOutgoing.h,v 1.3 2003/11/16 19:28:05 ullstar Exp $
 *
 * Copyright (C) 2003 Ulrich Abend
 *
 * This file is part of isdngw, a free isdn gateway plugin 
 * for sems, a free SIP media server.
 *
 * isdngw is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * For a license to use the ser software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * isdngw is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef _ISDNDEVICEOUTGOING_H_
#define _ISDNDEVICEOUTGOING_H_

#include "IsdnDevice.h" /* parent class */

/** return code if error occured */
#define DIALING_STATE_ERROR       FAIL
/** return code if pstn side is (probably...) ringing */
#define DIALING_STATE_RINGING     1
/** return code if busy signal is detected */
#define DIALING_STATE_BUSY        2
/** return code if connection is established */
#define DIALING_STATE_CONNECTED   3
/** return code if connection is established */
#define DIALING_STATE_NODIALTONE  4

/** An outgoing isdn device. 
 */
class IsdnDeviceOutgoing : public IsdnDevice
{
public:
	IsdnDeviceOutgoing(char* devicename);
//	~IsdnDeviceOutgoing();
	void cleanup();

	const char* getType() { return "OUTGOING"; };

	static IsdnDevice* getLockedDevice();

	void onError(unsigned int code, const string& reason);
	void onBye();
	
	void onBeforeCallAccept(AmRequest *req);
	
	/* check for numbers and so on then calls dial() */
	void placeCall(AmRequest *req, char* number, char* msn = NULL);
	
	/* wait and see what happens to proceeding call */
	int wait();
	
	/* establish voice connection, must be in state IS_CONNECTED */
	int connectVoice();
	
	private:
	/* trigger a call to number with specified msn or default msn */
	int dial(char* number, char* msn = NULL);
};

#endif
