/*
 * $Id: IsdnGatewayDialog.cpp,v 1.5 2004/01/23 12:25:55 ullstar Exp $
 *
 * Copyright (C) 2003 Ulrich Abend
 *
 * This file is part of isdngw, a free isdn gateway plugin 
 * for sems, a free SIP media server.
 *
 * isdngw is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * For a license to use the ser software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * isdngw is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "globals.h"
#include "IsdnGatewayDialog.h"
#include "IsdnConfiguration.h"
#include "IsdnDevice.h"
#include "IsdnChat.h"
#include "AmAudioIsdn.h"
#include "AmApi.h"
#include "AmSession.h"
#include "AmConfig.h"
#include "log.h"

/** Creates a new IsdnGatewayDialog. */
IsdnGatewayDialog::IsdnGatewayDialog(IsdnCall* call) 
	 : m_call (call)
{
	DBG("new IsdnGatewayDialog@%p (Call@%p)\n", this, call);
}

/** A connection exists. 
 */
void IsdnGatewayDialog::onSessionStart(AmRequest* req)
{
	//DBG("onSessionStart\n");
	if (m_call == NULL) {
		ERROR("internal error, call is NULL\n");
	} else {	
		m_call->setRequest(req);
		AmSession* s = getSession();
		if (s == NULL) {
			ERROR("fatal: session is NULL\n");
			exit(1);
		}	
		m_call->setSession(s);
		m_call->changeSIPState(CALLSTATE_CONNECTED);
	}	
}	

/** pre call setup, INVITE, OK, after this one ACK is sent! */
void IsdnGatewayDialog::onBeforeCallAccept(AmRequest* req)
{
	//DBG("onBeforeCallAccept\n");
	if (m_call == NULL) {
		ERROR("internal error, call is NULL\n");
	} else {	
		m_call->setRequest(req);
		m_call->changeSIPState(CALLSTATE_CONNECTING);
	}
}

/** error in call setup */
void IsdnGatewayDialog::onError(unsigned int code, const string& reason)
{
	//DBG("onerror called\n");
	if (m_call == NULL) {
		ERROR("internal error, call is NULL\n");
	} else {	
		m_call->changeSIPState(CALLSTATE_DISCONNECTED, code, reason.c_str());
	}	
}

/** An active connection is being terminated from SIP side. The voice 
 * connection must be terminated, a flag is being set to show that. 
 */
void IsdnGatewayDialog::onBye(AmRequest* req)
{
	//DBG("onByeCalled\n");
	if (m_call == NULL) {
		ERROR("internal error, call is NULL\n");
	} else {	
		m_call->changeSIPState(CALLSTATE_DISCONNECTED);
	}	
}

// TODO: remove all calls to this one
/** Hangs up the PSTN call and reintializes the isdn device. 
 * The lock on the device is removed. 
 */
/*
void IsdnGatewayDialog::hup(AmRequest* req)
{
	m_idev->hangup();
	if (m_idev->initialize() == FAIL) {
		ERROR("####### initialization failed, lost device, keep it locked\n");
	} else {	
		m_idev->unlock();	
	}	
}
*/

int IsdnGatewayDialog::onOther(AmSessionEvent* event) {
	ERROR("unimplemented IsdnGatewayDialog::onOther!!!\n");
	return 0;
}

IsdnGatewayDialog::~IsdnGatewayDialog()
{
	//DBG("destroying IsdnGatewayDialog!\n");
	delete m_call;
}

