<?php

define("FIFO", "/tmp/am_fifo");
define("FIFOVERSION", "0.2");
define("REPLYFIFOPREFIX", "/tmp/phpfifo_");
define("REPLYFIFOMODE", 0666);

/** calls the given command along with the array of parameters from isdngw via the 
 * sems fifo interface. This function returns an array containing the following 
 * fields:
 * "success": Whether the call resulted in OK (success=1) or ERROR (success=0).
 * "lines": Array of line is success=1
 * "reason": Error reason reported from sems or isdngw
 */
function doFifoCommand($command, $parameter) {
	if (!is_writable(FIFO)) {
		echo "ERROR: fifo " . FIFO . " not writeable.";
		return array ("success" => 0, "reason" => "FIFO not writeable");
	}	
	umask(0);
	$handle = fopen(FIFO, "w");
	fwrite ($handle, FIFOVERSION . "\n");
	fwrite ($handle, $command . "\n");
	$randomnumber = rand(11111, 9999999);
	$replyfilename = REPLYFIFOPREFIX . $randomnumber;
	posix_mkfifo($replyfilename, REPLYFIFOMODE);
	fwrite ($handle, $replyfilename . "\n");
	if (count($parameter) != 0) {
		$i = 0;
		foreach ( $parameter as $foo => $line ) {
			$i++;
			fwrite ($handle, $line . "\n");
		}	
	} else {
	// noop
	}
	fwrite ($handle, "\n");
	fclose ($handle);

	$replyhandle = fopen ($replyfilename, "r");
	$result = fread ($replyhandle, 2);
	if ($result == "OK") {
		$ok = true;
	} else if ( ($result == "ER") && (fread($replyhandle,3) == "ROR")) {
		$ok = false;
	}
	$result = fread ($replyhandle, 1);
	$result = fread ($replyhandle, 100000);

	fclose ($replyhandle);
	unlink ($replyfilename);

	if ($ok == true) {
		for ($i=0; $i<=strlen ($result); $i++) {
			if ($result{$i} != "\n") {
				$currentline .= $result{$i};
			} else {
				if ($currentline != "") {
					$lines[] = $currentline;
				}	
				$currentline = "";
			}
		}		
		
		if (count($lines) != 0)
			return array ("success" => 1, "lines" => array_values ($lines));
		else
			return array ("success" => 1);
	} else {
		return array ("success" => 0, "reason" => $result);
	}	
}


function getDetails ($result) {
	// to be returned:
	$returnvalue["status"] = "not/never<br>connected";
	$returnvalue["time"] = "00:00:00";

	$audio_start_time = time();
	$audio_end_time = time();

	for ($i=0; $i<count($result["lines"]); $i++) {
		$fields = explode (":", $result["lines"][$i]);
//		echo "<b>" . date("H:i:s",$fields[0]) . ": </b>";
		if ( $fields[1] == " AUDIO CONNECTED" ) {
			$audio_start_time = $fields[0];
			$returnvalue["status"] = "connected";
		} else if ( $fields[1] == " AUDIO DISCONNECTED" ) {
			$audio_end_time = $fields[0];
			$returnvalue["status"] = "done";
		} 
	}
	$seconds = ($audio_end_time-$audio_start_time);
	$minutes = $seconds / 60;
	$seconds = $seconds % 60;
	$hours   = floor( $minutes / 60 );
	$minutes = $minutes % 60;
	if ($seconds < 10) {
		$seconds = "0" . $seconds;
	};	
	if ($minutes < 10) {
		$minutes = "0" . $minutes;
	};	
	if ($hours < 10) {
		$hours = "0" . $hours;
	};	
	$returnvalue["time"] = $hours . ":" . $minutes . ":" . $seconds;
	return $returnvalue;
}

function display_row($fields) {
	echo "<tr><td>$fields[0]</td>";
	
	echo "<td>";
	if ($fields[1]=="PSTN2SIP")
		echo "<IMG src=img/phone.png height=32 width=32><IMG src=img/arrow.png height=32 width=32><IMG src=img/tux.png height=32 width=32>";
	else
		echo "<IMG src=img/tux.png height=32 width=32><IMG src=img/arrow.png height=32 width=32><IMG src=img/phone.png height=32 width=32>";
	echo "</td>";
	
	
	echo "<td>"; 
	echo "<IMG src=img/";
	if ($fields[2]=="CAPI")
		echo "capi.gif";
	else
		echo "i4l.gif";
	echo " height=32 width=83>";
	echo "</td>";
		
	
	echo "</td>";
	
	
	echo "<td><IMG src=img/device.png height=32 width=32 alt=device> $fields[3]</td><td><IMG src=img/phonefrom.png height=32 width=32 alt=fromnumber> $fields[4]</td><td><IMG src=img/phoneto.png height=32 width=32 alt=tonumber> $fields[5]</td>";
	$det = doFifoCommand("isdninfo.calldetails",array($fields[0]));
	if ($det["success"] == 1) {
		$datafield = getDetails($det);
		echo "<td>" . $datafield["time"] . "</td>";
		echo "<td>" . $datafield["status"] . "</td>";
	}
	
	echo "<td>";
	echo "<A href=calls.php?action=details&detail=" . $fields[0] . "><IMG alt=details src=img/details.png></A><A href=calls.php?action=delete&detail=" . $fields[0] . "><IMG src=img/delete.png alt='delete entrys'</A></tr>";
}

function list_header() {
	echo "<table border=1><tr><td>no.</td><td>direction</td><td>backend</td><td>device</td><td>from</td><td>to</td><td>time</td><td>status</td><td>actions</td></tr>";
}

?>
