/*
 * $Id: Ivr.cpp,v 1.16 2004/05/17 13:43:51 ilk Exp $
 * Copyright (C) 2002-2003 Fhg Fokus
 *
 * This file is part of sems, a free SIP media server.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "SemsConfiguration.h"
#include "log.h"
#include "AmApi.h"
#include "AmUtils.h"

#include "IvrPython.h"
#include "Ivr.h"

#include <unistd.h>

// is defined in Ivr.h #define MOD_NAME "ivr"

EXPORT_FACTORY(IvrFactory,MOD_NAME);

IvrFactory:: IvrFactory(const string& _app_name)
  : AmStateFactory(_app_name){
}

/**
 * Loads python script path and default script file from configuration file
 */
int IvrFactory::onLoad(){
	if(mIvrConfig.reloadModuleConfig(MOD_NAME) == 1){
      char* pSF = 0;
      if(((pSF = mIvrConfig.getValueForKey("python_script_path")) != NULL) && ( *pSF != '\0') ){
         setenv("PYTHONPATH",pSF,0);
         pythonScriptPath = pSF;
         if(pythonScriptPath.rfind('/') != pythonScriptPath.length() - 1)
            pythonScriptPath += "/";
      }
		else {
         pythonScriptPath = "/";
			WARN("no python_script_path in configuration\n");
		}
		if(((pSF = mIvrConfig.getValueForKey("python_script_file")) != NULL) && ( *pSF != '\0') ){
         defaultPythonScriptFile = pSF;
         return 0;
      }
		else {
			WARN("no python_script_file in configuration\n");
			return -1;
		}
   }
	return -1;
}

/**
 * chooses script according user name
 * if there is no special script default script will be used.
 */
AmDialogState* IvrFactory::onInvite(AmCmd& cmd){
   string pythonScriptFile = pythonScriptPath + cmd.domain	+ "/" + cmd.user + ".py";
   if(!file_exists(pythonScriptFile))
      pythonScriptFile = pythonScriptPath + cmd.user + ".py";
      if(!file_exists(pythonScriptFile))
	      pythonScriptFile = pythonScriptPath + defaultPythonScriptFile;
   return new IvrDialog(pythonScriptFile);
}


IvrDialog::IvrDialog(string scriptFile){
   pythonScriptFile = scriptFile;
}

IvrDialog::~IvrDialog(){}


void IvrDialog::onSessionStart(AmRequest* req){
   ivrPython = new IvrPython();
   ivrPython->fileName = (char*)pythonScriptFile.c_str();
   ivrPython->pAmSession = getSession();
   ivrPython->pCmd = &(req->cmd);
   ivrPython->start();
   AmThreadWatcher::instance()->add(ivrPython);
   sleep(1);                   
   while(!getSession()->sess_stopped.get() && !ivrPython->getStopped()){
         processEvents();
         sleep(1);
   }
   if(!getSession()->sess_stopped.get())
      req->bye();
   getSession()->sess_stopped.wait_for(); 
}


void IvrDialog::onBye(AmRequest* req){
   if( !ivrPython->getStopped())
      ivrPython->stop();
   DBG("now executing onBye\n");
}

int IvrDialog::onOther(AmSessionEvent* event)
{
    if(event->event_id == AmSessionEvent::Notify){
        if (strstr(event->request.getBody().c_str(),"200 OK") != NULL){
            ivrPython->isEvent.set(true);
        }
        event->processed = true;
        DBG("Notify event: body= %s\n",event->request.getBody().c_str());
        
    }

    return AmDialogState::onOther(event);
}
