/*
 * $Id: IvrDtmf.cpp,v 1.12 2004/06/15 10:09:19 ilk Exp $
 * Copyright (C) 2002-2003 Fhg Fokus
 *
 * This file is part of sems, a free SIP media server.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <math.h>
#include <sys/time.h>
#include "log.h"
#include "IvrDtmf.h"


float coeff[] = { 
   1.7077378035, 1.6452810764, 1.5686869621, 1.4782046080, 1.1641039848, 0.9963701963, 0.7986183763,
   0.9164258242, 0.7070164084, 0.4608556628, 0.1851755679, -0.6447557807, -1.0071394444, -1.3621084690
};

IvrDtmf::IvrDtmf(){}

IvrDtmf::~IvrDtmf(){}

bool IvrDtmf::calculate_power(pcm* data, float* power)
{
  float u0[NUMTONES],u1[NUMTONES],t,in;
  int i,j;
  pcm middle = 1<<(sizeof(pcm)*8 -1);
  for(j=0; j<NUMTONES; j++) {
    u0[j] = 0.0;
    u1[j] = 0.0;
  }
  for(i=0; i<NUMBER_OF_SAMPLES; i++) {   
    in = (float)data[i] / (float)middle; //for signed char 128.0;
    for(j=0; j<NUMTONES; j++) {
      t = u0[j];
      u0[j] = in + coeff[j] * u0[j] - u1[j];
      u1[j] = t;
    }
  }
  for(j=0; j<NUMTONES; j++)  
    power[j] = u0[j] * u0[j] + u1[j] * u1[j] - coeff[j] * u0[j]* u1[j];
  return(0);
}


int IvrDtmf::decode(pcm *data)
{
  float power[NUMTONES],thresh,thresh2nd,maxpower;
  int on[NUMTONES/2],on_count;
  int rcount, ccount;
  int row, col,i;
  int r[4],c[3];

  calculate_power(data,power);
  for(i=0, maxpower=0.0; i<NUMTONES;i++){
    if(power[i] > maxpower)
      maxpower = power[i];
      //printf("%f***", power[i]);
  }
  //printf("\n");
  if(maxpower < THRESH){  /* silence? */
    //DBG("Real silence %f \n", maxpower);
    return(DSIL);
  }
  thresh = RANGE * maxpower;    /* allowable range of powers */
  thresh2nd = thresh/5;
  for(i=0, on_count=0; i<NUMTONES/2; i++) {
    if(power[i] > thresh && power[i+NUMTONES/2] < thresh2nd) {    /* proof if frequence and its 2nd harmonic present*/
      on[i] = 1;
      on_count ++;
      //DBG("%i\n", i);
    } else
      on[i] = 0;
  }

  if(on_count == 2) {
/*    if(on[TON1] && on[TON2])
      return(DDT);
    if(on[TON2] && on[TON3])
      return(DRING);
    if(on[TON3] && on[TON4])
      return(DBUSY);
*/
    //c[0]= on[C1]; c[1]= on[C2]; c[2]= on[C3]; 
    //r[0]= on[R1]; r[1]= on[R2]; r[2]= on[R3]; r[3]= on[R4];
    c[0]=on[4]; c[1]=on[5]; c[2]= on[6];
    r[0]= on[0]; r[1]= on[1]; r[2]= on[2]; r[3]= on[3];

    for(i=0, rcount=0; i<4; i++) {
      if(r[i]) {
        rcount++;
        row = i;
      }
    }
    for(i=0, ccount=0; i<3; i++) {
      if(c[i]) {
        ccount++;
        col = i;
      }
    }
    if(rcount==1 && ccount==1) {   //DTMF
        if(row == 3 && col == 0 )
           return(DSTAR);
        if(row == 3 && col == 2 )
           return(DNUM);
        if(row == 3)
           return(D0);
        if(row == 0 && col == 2) {   /* DTMF 3 conflicts with MF 7 */
            return(D3);
        } else
          return(D1 + col + row*3);
    }
    return(-1);
  }
  if(on_count == 0)
    return(DSIL);
  return(-1);
}

// Example:
int IvrDtmf::get_frame(pcm* buf)
{
/*  int i,x;
  extern FILE* fd;
  for(i=0; i<NUMBER_OF_SAMPLES; ) {
    x = fread(&buf[i], sizeof(pcm), NUMBER_OF_SAMPLES-i, fd);
    if(x <= 0)
      return(0);
    i += x;
  }
  return(1);
*/
return 0;
}


int IvrDtmf::getNumber()
{
  int x,last= DSIL;
  pcm frame[NUMBER_OF_SAMPLES<+5];
  int silence_time;

  while(get_frame(frame)) {
    if ( sizeof(frame) < NUMBER_OF_SAMPLES*sizeof(pcm)){
      DBG("IvrDtmf::getNumber: frame is too small ( <240 )");
      return 0;
    }
    x = decode(frame);
    if(x >= 0) {
      if(x == DSIL)
        silence_time += (silence_time>=0)?1:0 ;
      else
        silence_time= 0;
      if(silence_time == FLUSH_TIME) {
        silence_time= -1; // stops counting 
      }
      if(x != DSIL && x != last &&
         (last == DSIL || last==D24 || last == D26 ||
          last == D2426 || last == DDT || last == DBUSY ||
          last == DRING) )  {
            return x;
      }
      last = x;
    }
  }
  return -1;//Not detected
}

//Timeout in milliseconds
//Detect DTMF signal and returns key
int IvrDtmf::getKey(unsigned int timeout)
{
   int x,last= -1; //DSIL  It can be noise, better detect DSIL first.
   pcm frame[NUMBER_OF_SAMPLES+5];
   int silence_time;
   long timediff = 0;
   timeval tvStart, tvNow;

   gettimeofday(&tvStart,0); 
   while(timediff < (long)timeout*1000){
      int ret;
      ret = get_frame(frame);
      if(ret > 0){
         x = decode(frame);
         if(x >= 0) {
            if(x == DSIL)
               silence_time += (silence_time>=0)?1:0 ;
            else
               silence_time= 0;
            if(silence_time == FLUSH_TIME) {
               silence_time= -1; // stops counting
            }
            // x < 16 added to detect only DTMF tones
            if(x != DSIL && x != last && x < 16 &&
               (last == DSIL || last==D24 || last == D26 ||
               last == D2426 || last == DDT || last == DBUSY ||
               last == DRING) )  {
                  return x;
            }
            last = x;
         }
      }
      else if(ret < 0)
         return ret;
      gettimeofday(&tvNow,0);
      timediff = (tvNow.tv_sec - tvStart.tv_sec)* 1000000 + ((tvNow.tv_usec > tvStart.tv_usec) ? (tvNow.tv_usec - tvStart.tv_usec) : (tvNow.tv_usec + 1000000 - tvStart.tv_usec));
   }
   return -1; //Not detected
}
