/*
 * $Id: IvrPython.cpp,v 1.22 2004/05/11 07:39:08 ilk Exp $
 * Copyright (C) 2002-2003 Fhg Fokus
 *
 * This file is part of sems, a free SIP media server.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <Python.h>
#include "log.h"
#include "AmApi.h"
#include "Ivr.h"
#include "IvrDtmf.h"
#include "AmAudioIvr.h"
#include "IvrPython.h"

#include <unistd.h>


//AmSession* amSession;
//IvrPython* pIvrPython;
//IvrDetect* pIvrDetect;
//int detKey;

//pthread_mutex_t mutexAmSession;



/***********************************************************************************************************
 *   Python  extensions
 *
 ***********************************************************************************************************
 */
extern "C" {
/**
 * Python extention for file playing
 */

static IvrPython* getIvrPythonPointer(){
   IvrPython* pIvrPython = NULL;
   PyObject *module = PyImport_ImportModule(PY_MOD_NAME);
   if (module != NULL) {
      PyObject *ivrPythonPointer = PyObject_GetAttrString(module, "ivrPythonPointer");
      if (ivrPythonPointer != NULL){
         if (PyCObject_Check(ivrPythonPointer))
            pIvrPython = (IvrPython*)PyCObject_AsVoidPtr(ivrPythonPointer);
            Py_DECREF(ivrPythonPointer);
         }
   }
   return pIvrPython;   
}

static PyObject* ivrPlay(PyObject *self, PyObject *args){
	char* fileName;
   IvrPython* pIvrPython = getIvrPythonPointer();
   if(pIvrPython != NULL){  
      if(PyArg_ParseTuple(args,"s", &fileName)){
         string sFileName(fileName);
		   DBG("IVR: start play invitation...(%s)\n",fileName);
         Py_BEGIN_ALLOW_THREADS
		   pIvrPython->pAmSession->rtp_str.play(sFileName);
         Py_END_ALLOW_THREADS
		   DBG("IVR: ...stop play invitation (%s).\n",fileName);
		   return Py_BuildValue("i",1);
	   }
	   else
		   return PyString_FromString("IVR Python Error: No Argument!");
    }
    else
      return PyString_FromString("IVR Python Error: Wrong pointer to IvrPython!");
}

/**
 * Python extention for file recording
 */
static PyObject* ivrRecord(PyObject *self, PyObject *args){
   char* fileName;
   unsigned int timeout;
   
   IvrPython* pIvrPython = getIvrPythonPointer();
   if(pIvrPython != NULL){ 
      if(PyArg_ParseTuple(args,"si", &fileName, &timeout)){
         string sFileName(fileName);
         DBG("IVR: start record message...(%s)\n",fileName);
         int recSize;
         Py_BEGIN_ALLOW_THREADS
         recSize = pIvrPython->pAmSession->rtp_str.record(sFileName, timeout );
         Py_END_ALLOW_THREADS
         DBG("IVR: ...stop record message (%s).\n",fileName);
         return Py_BuildValue("i",recSize);
      }
      else
         return PyString_FromString("IVR Python Error: No Argument!");
   }
   else
      return PyString_FromString("IVR Python Error: Wrong pointer to IvrPython!");
}


/**
 * Python extention for file playing & DTMF detection
 */
static PyObject* ivrPlayAndDetect(PyObject *self, PyObject *args){
	char* fileName;
   unsigned int timeout;

   IvrPython* pIvrPython = getIvrPythonPointer();
   if(pIvrPython != NULL){ 
	   if(PyArg_ParseTuple(args,"si", &fileName, &timeout)){
         string sFileName(fileName);
         AmAudioFile* pAmAudioFile;
         pAmAudioFile = new AmAudioFile("Wav");
         pAmAudioFile->loop.set(true);
         IvrDetect* ivrDetect = new IvrDetect(timeout);
         ivrDetect->pAmAudioIvr = new AmAudioIvr();
         ivrDetect->pAmSession = pIvrPython->pAmSession;
         if(pAmAudioFile->open(sFileName.c_str(),AmAudioFile::Read)){
            ERROR("IVR Python Error: Wrong file name!");
            PyErr_SetString(PyExc_StandardError,"IVR Python Error: Wrong file name!");
            return Py_BuildValue("i", -1);
         }
         int detectedKey;
         ivrDetect->start();
         AmThreadWatcher::instance()->add(ivrDetect);
         DBG("start duplex\n");
         Py_BEGIN_ALLOW_THREADS
         pIvrPython->pAmSession->rtp_str.duplex(pAmAudioFile,ivrDetect->pAmAudioIvr);
         Py_END_ALLOW_THREADS
         ivrDetect->pAmAudioIvr->close();
         pAmAudioFile->close();
         detectedKey = ivrDetect->getDetectedKey();
         pIvrPython->pAmSession->rtp_str.resume();

         if(detectedKey == -2)
            pIvrPython->stop();
         //pIvrPython->pAmAudioFile->close();
         DBG("%i detected\n",detectedKey);
         return Py_BuildValue("i", detectedKey);
      }
	   else
         return PyString_FromString("IVR Python Error: No Argument");
   }
   else
      return PyString_FromString("IVR Python Error: Wrong pointer to IvrPython!");         
}


/**
 * Python extention for DTMF detection
 */
static PyObject* ivrDetect(PyObject *self, PyObject *args){
   unsigned int timeout;   //in millisecons

   IvrPython* pIvrPython = getIvrPythonPointer();
   if(pIvrPython != NULL){ 
      if(PyArg_ParseTuple(args,"i", &timeout)){
         IvrDetect* ivrDetect = new IvrDetect(timeout);
         ivrDetect->pAmAudioIvr = new AmAudioIvr();
         ivrDetect->pAmSession = pIvrPython->pAmSession;
         int detectedKey;
         ivrDetect->start();
         AmThreadWatcher::instance()->add(ivrDetect);
         DBG("start record to detector\n");
         Py_BEGIN_ALLOW_THREADS
         pIvrPython->pAmSession->rtp_str.record(ivrDetect->pAmAudioIvr,(int)timeout/1000  + 1 ); //in seconds   +1 while detect will control time
         Py_END_ALLOW_THREADS
         ivrDetect->pAmAudioIvr->close();
         detectedKey = ivrDetect->getDetectedKey();
         pIvrPython->pAmSession->rtp_str.resume();

         if(detectedKey == -2)
            pIvrPython->stop();
         DBG("%i detected\n",detectedKey);
         return Py_BuildValue("i", detectedKey);
	   }
	   else
		   return PyString_FromString("IVR Python Error: No Argument!!!");
   }
   else
      return PyString_FromString("IVR Python Error: Wrong pointer to IvrPython!");
}


/**
 * Python extention for getting time
 */
static PyObject* ivrGetTime(PyObject *self, PyObject *args){
   return Py_BuildValue("i",(int)time(NULL));
}


/**
 * Python extention for caller name
 */
static PyObject* ivrGetFrom(PyObject *self, PyObject *args){
   IvrPython* pIvrPython = getIvrPythonPointer();
   if(pIvrPython != NULL){ 
      return Py_BuildValue("s",pIvrPython->pCmd->from.c_str());
   }
   else
      return PyString_FromString("IVR Python Error: Wrong pointer to IvrPython!");
}


/**
 * Python extention for callee name
 */
static PyObject* ivrGetTo(PyObject *self, PyObject *args){
   IvrPython* pIvrPython = getIvrPythonPointer();
   if(pIvrPython != NULL){ 
      return Py_BuildValue("s",pIvrPython->pCmd->to.c_str());
   }
   else
      return PyString_FromString("IVR Python Error: Wrong pointer to IvrPython!");
}


/**
* Python extention for caller uri
*/
static PyObject* ivrGetFromURI(PyObject *self, PyObject *args){
   IvrPython* pIvrPython = getIvrPythonPointer();
   if(pIvrPython != NULL){
      return Py_BuildValue("s",pIvrPython->pCmd->from_uri.c_str());
   }
   else
      return PyString_FromString("IVR Python Error: Wrong pointer to IvrPython!");
}


/**
 * Python extention for callee uri
 */
static PyObject* ivrGetToURI(PyObject *self, PyObject *args){
   IvrPython* pIvrPython = getIvrPythonPointer();
   if(pIvrPython != NULL){
      return Py_BuildValue("s",pIvrPython->pCmd->r_uri.c_str());
   }
   else
      return PyString_FromString("IVR Python Error: Wrong pointer to IvrPython!");
}


/**
 * Python extention for domain name
 */
static PyObject* ivrGetDomain(PyObject *self, PyObject *args){
   IvrPython* pIvrPython = getIvrPythonPointer();
   if(pIvrPython != NULL){
      return Py_BuildValue("s",pIvrPython->pCmd->domain.c_str());
   }
   else
      return PyString_FromString("IVR Python Error: Wrong pointer to IvrPython!");
}

   
/**
 * Python extention for redirection
 */
static PyObject* ivrRedirect(PyObject *self, PyObject *args){
   char* uri;

   IvrPython* pIvrPython = getIvrPythonPointer();
   if(pIvrPython != NULL){
      if(PyArg_ParseTuple(args,"s", &uri)){
         string refer_to(uri);
         Py_BEGIN_ALLOW_THREADS
         AmCmd refer_cmd = AmRequestUAC::refer(*pIvrPython->pCmd,refer_to);
         Py_END_ALLOW_THREADS
         // was send
         if(pIvrPython->waitForEvent(30000))
            return Py_BuildValue("i",1);
      }
      // error
      DBG("Redirect timeout\n");
      return Py_BuildValue("i",0);
   }
   else
      return PyString_FromString("IVR Python Error: Wrong pointer to IvrPython!");
}

} // end of extern "C"


/***********************************************************************************************************
 *   IvrDetect class functions
 *
 ***********************************************************************************************************
*/

IvrDetect::IvrDetect(unsigned int detectTimeout)
{
   timeout = detectTimeout;
}
int IvrDetect::getDetectedKey(){
   return detectedKey;
}

void IvrDetect::run()
{
   DBG("Start DTMF detection\n");
   detectedKey = getKey(timeout);
   pAmSession->rtp_str.pause();
}


void IvrDetect::on_stop()
{
}

int IvrDetect::get_frame(pcm* buf){
   return pAmAudioIvr->getAudioFrame((unsigned char*)buf);
}



/***********************************************************************************************************
 *   IvrPython class functions
 *
 ***********************************************************************************************************
*/


IvrPython::IvrPython()
    : isEvent(false)
{
}

IvrPython::~IvrPython(){

}

void IvrPython::run(){
   FILE* fp;
   int retval;
   static PyThreadState* pyMainThreadState;
   fp = fopen((char*)fileName,"r");
	if(fp != NULL){
      PyMethodDef extIvrPython[] = {
         {"play", ivrPlay, METH_VARARGS, "Example Module"},
         {"record", ivrRecord, METH_VARARGS, "Example Module"},
         {"playAndDetect", ivrPlayAndDetect, METH_VARARGS, "Example Module"},
         {"detect", ivrDetect, METH_VARARGS, "Example Module"},
         {"getTime", ivrGetTime, METH_VARARGS, "Example Module"},
         {"getFrom", ivrGetFrom, METH_VARARGS, "Example Module"},
         {"getTo", ivrGetTo, METH_VARARGS, "Example Module"},
         {"redirect", ivrRedirect, METH_VARARGS, "Example Module"},
         {"getFromURI", ivrGetFromURI, METH_VARARGS, "Example Module"},
         {"getToURI", ivrGetToURI, METH_VARARGS, "Example Module"},
         {"getDomain", ivrGetDomain, METH_VARARGS, "Example Module"},
         {NULL, NULL, 0, NULL},
      };
      
      
      if(!Py_IsInitialized()){
         DBG("Start Python\n");
         Py_Initialize();
         PyEval_InitThreads();
         pyMainThreadState = PyEval_SaveThread();         
      }       
      DBG("Start new Python interpreter\n");
      PyEval_AcquireLock();
      PyThreadState* pyThreadState;      
      if ( (pyThreadState = Py_NewInterpreter()) != NULL){

         PyObject* ivrPyInitModule = Py_InitModule(PY_MOD_NAME, extIvrPython);
         PyObject* ivrPythonPointer = PyCObject_FromVoidPtr((void*)this,NULL);
         if (ivrPythonPointer != NULL)
            PyModule_AddObject(ivrPyInitModule, "ivrPythonPointer", ivrPythonPointer);
     
         if(!PyRun_SimpleFile(fp,(char*)fileName)){
            fclose(fp);
            retval = 0;// true;
         }
         else{
            PyErr_Print();
            ERROR("IVR Python Error: Failed to run \"%s\"\n", (char*)fileName);
		      retval = -1;// false;
	      }


         Py_EndInterpreter(pyThreadState);

      }
      else{
         ERROR("IVR Python Error: Failed to start new intrepreter\n");
      }
      PyEval_ReleaseLock();
   }
   else{
      ERROR("IVR Python Error: Can not open file \"%s\"\n",(char*) fileName);
      retval = -1;// false;
   }
   DBG("End Python\n");
}

void IvrPython::on_stop()
{
}

int IvrPython::waitForEvent(unsigned int timeout){
   unsigned int timediff = 0;
   int ret = 0;
   timeval tvStart, tvNow;
   Py_BEGIN_ALLOW_THREADS
   gettimeofday(&tvStart,0);
   while(timediff < timeout*1000){
      if(isEvent.get()){
         isEvent.set(false);
         ret = 1;
         break;
      }
      gettimeofday(&tvNow,0);
      timediff = (tvNow.tv_sec - tvStart.tv_sec)* 1000000 + (tvNow.tv_usec - tvStart.tv_usec);
      sleep(1);
   }
   Py_END_ALLOW_THREADS
   return ret; // timeout
}





