/*
 * $Id: NumberReader.cpp,v 1.3 2003/11/25 16:19:19 rco Exp $
 *
 * Copyright (C) 2002-2003 Fhg Fokus
 *
 * This file is part of sems, a free SIP media server.
 *
 * sems is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * For a license to use the sems software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * sems is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "NumberReader.h"
#include "SemsConfiguration.h"
#include "AmUtils.h"
#include "log.h"

#include <unistd.h>

#define MODULE_NAME "number_reader"


EXPORT_FACTORY(NumberReaderFactory,MODULE_NAME);

string NumberReaderFactory::number_path;
string NumberReaderFactory::prolog_file;
string NumberReaderFactory::epilog_file;

int NumberReaderFactory::onLoad()
{
    char* temp;
    SemsConfiguration conf;
    if(!conf.reloadModuleConfig(MODULE_NAME)){
	ERROR("Number reader configuration didn't load.");
	return -1;
    }

    temp = conf.getValueForKey("number_path");
    if (temp == NULL) {
	ERROR("required parameter number_path missing\n");
	return -1;
    }	
    number_path = temp;


    temp = conf.getValueForKey("prolog_file");
    if (temp == NULL) {
	ERROR("required parameter prolog_file missing\n");
	return -1;
    }	
    prolog_file = temp;
    prolog_file = number_path + "/" + prolog_file;
    if(!file_exists(prolog_file)){
	ERROR("prolog file '%s' does not exist.\n",prolog_file.c_str());
    }

    temp = conf.getValueForKey("epilog_file");
    if (temp == NULL) {
	ERROR("required parameter epilog_file missing\n");
	return -1;
    }	
    epilog_file = temp;
    epilog_file = number_path + "/" + epilog_file;
    if(!file_exists(epilog_file)){
	ERROR("epilog file '%s' does not exist.\n",epilog_file.c_str());
    }

    for(int i=0; i<10; i++){
	string f = number_path + "/" + int2str(i) + ".wav";
	if(!file_exists(f)){
	    ERROR("Number reader could not find %s.\n",f.c_str());
	    return -1;
	}
    }

    return 0;
}


NumberReaderFactory::NumberReaderFactory(const string& app_name)
    : AmStateFactory(app_name)
{
}

AmDialogState* NumberReaderFactory::onInvite(AmCmd& cmd)
{
    string number = cmd.user;
    for(string::iterator it = number.begin();
	  it != number.end(); ++it){
	
	if((*it < '0') || (*it > '9')){
	    ERROR("Number reader: wrong number: %s\n",number.c_str());
	    return NULL;
	}
    }

    return new NumberReaderDialog(number);
}

NumberReaderDialog::NumberReaderDialog(const string& number)
    : number(number)
{
}

NumberReaderDialog::~NumberReaderDialog()
{
}

void NumberReaderDialog::onSessionStart(AmRequest* req)
{
    try {
	DBG("starting sending numbers...(%s)\n",req->cmd.user.c_str());
	sleep(1);
	AmRtpStream& rtp_str = getSession()->rtp_str;
	rtp_str.play(NumberReaderFactory::prolog_file.c_str());

	for(string::iterator it = req->cmd.user.begin();
	    it != req->cmd.user.end(); ++it) {

	    rtp_str.play(NumberReaderFactory::number_path + "/" + *it + ".wav");
	}

	rtp_str.play(NumberReaderFactory::epilog_file.c_str());
	DBG("... number reader reaches the end.\n");

	if(!getSession()->sess_stopped.get())
	    req->bye();
    }
    catch(const string& err){
	ERROR("%s\n",err.c_str());
    }
}

void NumberReaderDialog::onBye(AmRequest* req)
{
}
