/*
 * Semstalk - Festival TTS in SEMS
 *    2003 by Stefan Sayer 
 *
 * Copyright (C) 2003 Fhg Fokus
 *
 * This file is part of sems, a free SIP media server.
 *
 * sems is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * For a license to use the ser software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * sems is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "Semstalk.h"
#include "AmConfig.h"
#include "AmUtils.h"

#include "sems.h"
#include "log.h"

#include "festival_client.h"

#define MOD_NAME "semstalk"

#include <unistd.h>
#include <fstream>

#define DEFAULT_CACHE_PATH "/tmp/"

using namespace std;

EXPORT_FACTORY(SemstalkFactory,MOD_NAME);


// festival server
/* For server/client */
int festival_socket_client(const char *host,int port);

string SemstalkFactory::FestivalServer;
string SemstalkFactory::FestivalPort;
string SemstalkFactory::FestivalVoice;
bool   SemstalkFactory::Caching;
string SemstalkFactory::CachePath;

SemstalkFactory::SemstalkFactory(const string& _app_name)
  : AmStateFactory(_app_name)
{
}

int SemstalkFactory::onLoad()
{
  char* p = 0;

  if(m_config.reloadModuleConfig(MOD_NAME) != 1) {
    WARN("no configuration specified for module ttsannounce.\n");
    WARN("using default configuration:\n");
    FestivalServer = "localhost";
    FestivalPort = "1314";
    FestivalVoice = "rab_diphone";
    Caching = true;
    CachePath = DEFAULT_CACHE_PATH;
    WARN("FestivalServer = \"%s\"\n", FestivalServer.c_str());      
    WARN("FestivalPort = \"%s\"\n", FestivalPort.c_str());
    WARN("FestivalVoice = \"%s\"\n", FestivalVoice.c_str());
    WARN("Caching = true\n");
    WARN("CachePath = \"%s\"\n", CachePath.c_str());
  } else { // configuring...

    if( ((p = m_config.getValueForKey("festival_server")) != NULL) && (*p != '\0') ) {
      FestivalServer = p;
    } else {
      WARN("no festival server specified in configuration\n");
      WARN("file for module semstalk.\n");
      FestivalServer = "localhost";
    }

    if( ((p = m_config.getValueForKey("festival_port")) != NULL) && (*p != '\0') )
      FestivalPort = p;
    else {
      WARN("no festival_port specified in configuration\n");
      WARN("file for module semstalk.\n");
      FestivalPort = "1314";
    }

    if( ((p = m_config.getValueForKey("festival_voice")) != NULL) && (*p != '\0') )
      FestivalVoice = p;
    else {
      WARN("no festival_voice specified in configuration\n");
      WARN("file for module semstalk.\n");
      FestivalVoice = "rab_diphone";
    }
  

    if( ((p = m_config.getValueForKey("caching")) != NULL) && (*p != '\0') ) {
      Caching = ((*p=='y') || (*p=='Y'));
    } else {
      WARN("no caching (y/n) specified in configuration\n");
      WARN("file for module ttsannouncement.\n");
      Caching = true;
    }

    if( ((p = m_config.getValueForKey("cache_path")) != NULL) && (*p != '\0') )
      CachePath = p;
    else {
      WARN("no cache_path specified in configuration\n");
      WARN("file for module ttsannouncement.\n");
      CachePath = DEFAULT_CACHE_PATH;
    }
    if( !CachePath.empty() 
	&& CachePath[CachePath.length()-1] != '/' )
      CachePath += "/";
  }
  return 0;
}

AmDialogState* SemstalkFactory::onInvite(AmCmd& cmd)
{
    return new SemstalkDialog(FestivalServer, FestivalPort, FestivalVoice, Caching, CachePath);
}

SemstalkDialog::SemstalkDialog(const string& festival_server, const string& festival_port, 
			       const string& FestivalVoice,const bool& Caching, const string& CachePath )
  : festival_server(festival_server), festival_port(festival_port), festival_voice(FestivalVoice), 
    caching(Caching), cache_path(CachePath)
{
}

SemstalkDialog::~SemstalkDialog()
{
}

void SemstalkDialog::onSessionStart(AmRequest* req)
{
    try {
       	string msg_filename = string("/tmp/") + req->cmd.callid + string(".wav");
	string uri_message = req->cmd.r_uri;
	if (!uri_message.length())
	  return;
	string message = "";  // a bit complicated but has all eventualities...
	for (string::iterator i = uri_message.begin()+uri_message.find(":")+1; i!=uri_message.end();i++) {
	  if (*i=='@')
	    break;
	  else if (*i == '%') {
	    string::iterator i1 = i+1;
	    string s;
	    while ((*i1 >='0')&&(*i1<='9')&&(i1!=message.end())) {
	      s+=*i1;
	      i1++;
	    }
	    char n[2]; 
	    char* endp;
	    n[0] = (char)strtoul(s.c_str(), &endp, 16); // hex encoding of %xx part
	    n[1] = 0;
	    message += n;
	    i=i1;
	  }	
	  message += *i;
	}

	DBG("starting tts of uri part \"%s\"...\n",message.c_str());
		
	string cache_filename = cache_path + message + string(".wav");
	if (caching) {
	  DBG(" trying cache \"%s\" .. ", cache_filename.c_str());
	  if (file_exists(cache_filename)) {
	    DBG("hit. Playing from cache.\n");
	    getSession()->rtp_str.play(cache_filename.c_str());
	    if(!getSession()->sess_stopped.get())
	      req->bye();
	    return;
	  } else {
	    DBG("miss.\n");
	  }
	}

	FT_Info *info = festival_default_info();
	info->server_host =  strdup(festival_server.c_str());
	info->server_port = atoi(festival_port.c_str());
	info->text_mode = "fundamental";
	
	info = festivalOpen(info);

	//------------------------------------------
	char* text = strdup(message.c_str());
	char *wave;
	FILE *fd;
	char *p;
	char ack[4];
	int n;
	int size;

	if (info == 0)
	    return;

	if (info->server_fd == -1)
	{
	    DBG("festival_client: server connection unopened\n");
	    return;
	}
	fd = fdopen(dup(info->server_fd),"wb"); 
	std::vector<std::string> commands;
	commands.push_back(std::string("(tts_return_to_client)\n"));
	commands.push_back(std::string("(voice_"+festival_voice+")\n"));
	// resampling in siteinit.scm does not work for don 
	if (festival_voice == "don_diphone") {
	  commands.push_back(std::string("(define (my_resample utter) (utt.wave.resample utter 8000) )\n"));
	  commands.push_back(std::string("(set! after_synth_hooks my_resample)\n")); 
	}
	commands.push_back("(audio_mode 'async)");
	std::string c("(tts_textall \"\n");
	for (p=text; p && (*p != '\0'); p++)
	{
	    if ((*p == '"') || (*p == '\\'))
	      c = c + (char)'\\';
	    c = c+(char)*p;
	}
	c = c + std::string("\"\"fundamental\")\n");
	commands.push_back(c);

	for (std::vector<std::string>::iterator i =commands.begin();i!=commands.end();i++) {
	  fprintf(fd, i->c_str());
	  fflush(fd);
	  wave = 0;
	  do {
	    for (n=0; n < 3; )
		n += read(info->server_fd,ack+n,3-n);
	    ack[3] = '\0';

	    if (strcmp(ack,"WV\n") == 0)  {        /* receive a waveform */
	      // TODO: instantly forward wave in rtp_str
		wave = client_accept_riff(info->server_fd, &size);
		if (wave){
		  save_RIFF(wave, size, msg_filename.c_str());	      
		  if (caching)
		    save_RIFF(wave, size, cache_filename.c_str());
		  getSession()->rtp_str.play(msg_filename.c_str());
		  fclose(fd); 
		  festivalClose(info);
		  if(!getSession()->sess_stopped.get())
		    req->bye();
		  return;
		}
	    }
	    else if (strcmp(ack,"LP\n") == 0) {   /* receive an s-expr */
		char* s_exp = client_accept_s_expr(info->server_fd);
		if (std::string(s_exp).substr(0,3) == "nil") { // signals end in tts_text
		  fclose(fd); 
		  festivalClose(info);
		  if(!getSession()->sess_stopped.get())
		    req->bye();
		  return;
		}
	    }
	    else if (strcmp(ack,"ER\n") == 0)    /* server got an error */
	    {
		ERROR("festival server returned error\n");
		break;
	    }
	  } while (strcmp(ack,"OK\n") != 0);
	} 



      }
      catch(const string& err){
	    ERROR("%s\n",err.c_str());
   }
}

void SemstalkDialog::onBye(AmRequest* req)
{
}

