/*
 * Semstalkflite - Flite TTS in SEMS
 *    2003 by Stefan Sayer 
 *
 * Copyright (C) 2003 Fhg Fokus
 *
 * This file is part of sems, a free SIP media server.
 *
 * sems is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * For a license to use the ser software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * sems is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "Semstalkflite.h"
#include "AmConfig.h"
#include "AmUtils.h"

#include "sems.h"
#include "log.h"

#define MOD_NAME "semstalkflite"

#define CACHE_PATH "/tmp/"

#include <unistd.h>
#include <fstream>

EXPORT_FACTORY(SemstalkfliteFactory,MOD_NAME);

extern "C" cst_voice *register_cmu_us_kal();
 
bool   SemstalkfliteFactory::Caching;
string SemstalkfliteFactory::CachePath;

SemstalkfliteFactory::SemstalkfliteFactory(const string& _app_name)
  : AmStateFactory(_app_name)
{
}

int SemstalkfliteFactory::onLoad()
{
  char* p = 0;

  if(m_config.reloadModuleConfig(MOD_NAME) != 1) {
    WARN("no configuration specified for module semstalkflite.\n");
    WARN("using default configuration:\n");
    Caching = "true";
    CachePath = "/tmp/";
    WARN("Caching = true\n");
    WARN("CachePath = \"%s\"\n", CachePath.c_str());
  } else { // configuring...

    if( ((p = m_config.getValueForKey("caching")) != NULL) && (*p != '\0') ) {
      Caching = ((*p=='y') || (*p=='Y'));
    } else {
      WARN("no caching (y/n) specified in configuration\n");
      WARN("file for module semstalkflite.\n");
      Caching = true;
    }

    if( ((p = m_config.getValueForKey("cache_path")) != NULL) && (*p != '\0') )
      CachePath = p;
    else {
      WARN("no cache_path specified in configuration\n");
      WARN("file for module semstalkflite.\n");
      CachePath = CACHE_PATH;
    }
    if( !CachePath.empty() 
	&& CachePath[CachePath.length()-1] != '/' )
      CachePath += "/";
  }
  return 0;
}

AmDialogState* SemstalkfliteFactory::onInvite(AmCmd& cmd)
{
    return new SemstalkfliteDialog(Caching, CachePath);
}

SemstalkfliteDialog::SemstalkfliteDialog(const bool& Caching , const string& CachePath)
  : caching(Caching), cache_path(CachePath)
{
    flite_init();
    v = register_cmu_us_kal();
}

SemstalkfliteDialog::~SemstalkfliteDialog()
{
}

void SemstalkfliteDialog::onSessionStart(AmRequest* req)
{
  try {
    string msg_filename = string("/tmp/") + req->cmd.callid + string(".wav");
    string uri_message = req->cmd.r_uri;
    if (!uri_message.length())
      return;
    string message = "";  // a bit complicated but has all eventualities...
    for (string::iterator i = uri_message.begin()+uri_message.find(":")+1; i!=uri_message.end();i++) {
      if (*i=='@')
	break;
      else if (*i == '%') {
	string::iterator i1 = i+1;
	string s;
	while ((*i1 >='0')&&(*i1<='9')&&(i1!=message.end())) {
	  s+=*i1;
	  i1++;
	}
	char n[2]; 
	char* endp;
	n[0] = (char)strtoul(s.c_str(), &endp, 16); // hex encoding of %xx part
	n[1] = 0;
	message += n;
	i=i1;
      }	
      message += *i;
    }

    DBG("starting tts of uri part \"%s\"...\n",message.c_str());

    // check caching/cached
    string cache_filename = cache_path + message + string(".wav");
    if (caching) {
      DBG(" trying cache \"%s\" .. ", cache_filename.c_str());
      if (file_exists(cache_filename)) {
	DBG("hit. Playing from cache.\n");
	getSession()->rtp_str.play(cache_filename.c_str());
	if(!getSession()->sess_stopped.get())
	  req->bye();
	return;
      } else {
	DBG("miss.\n");
      }
    }
    //-----------------
    cst_wave *w;
    cst_utterance *u;
    u = flite_synth_text(message.c_str(),v);
    w = utt_wave(u);
    float durs;
    durs = (float)w->num_samples/(float)w->sample_rate;
	
    cst_wave_save_riff(w,msg_filename.c_str());
    if (caching)
      cst_wave_save_riff(w, cache_filename.c_str());
    delete_utterance(u);
    DBG("%f seconds of speech synthesized\n",durs);
    //-----------------

    getSession()->rtp_str.play(msg_filename.c_str());
    if(!getSession()->sess_stopped.get()) {
      DBG("sending bye...");
      req->bye();
    }
    DBG("done.");
  }
  catch(const string& err){
    ERROR("%s\n",err.c_str());
  }
}

void SemstalkfliteDialog::onBye(AmRequest* req)
{
}

