/*
 * TTSAnnflite - FLite TTS Announcement in SEMS
 *    2003 by Stefan Sayer 
 *
 * Copyright (C) 2003 Fhg Fokus
 *
 * This file is part of sems, a free SIP media server.
 *
 * sems is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * For a license to use the ser software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * sems is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "TTSAnnflite.h"
#include "AmConfig.h"
#include "AmUtils.h"

#include "sems.h"
#include "log.h"

#define MOD_NAME "ttsannflite"

#include <unistd.h>
#include <fstream>

// these defaults are used if no config is given
#define DEFAULT_CACHE_PATH "/tmp/"
#define DEFAULT_ANNOUNCE_PATH "texts/"
#define DEFAULT_ANNOUNCE_FILE "default.txt"

using namespace std; 

EXPORT_FACTORY(TTSAnnfliteFactory,MOD_NAME);

extern "C" cst_voice *register_cmu_us_kal();

string TTSAnnfliteFactory::AnnouncePath;
string TTSAnnfliteFactory::AnnounceFile;
bool   TTSAnnfliteFactory::Caching;
string TTSAnnfliteFactory::CachePath;

TTSAnnfliteFactory::TTSAnnfliteFactory(const string& _app_name)
  : AmStateFactory(_app_name)
{
}

int TTSAnnfliteFactory::onLoad()
{
    char* p = 0;

    if(m_config.reloadModuleConfig(MOD_NAME) != 1) {
      WARN("no configuration specified for module ttsannflite.\n");
      WARN("using default configuration:\n");
      AnnouncePath = DEFAULT_ANNOUNCE_PATH;
      AnnounceFile = DEFAULT_ANNOUNCE_FILE;
      Caching = true;
      CachePath = DEFAULT_CACHE_PATH;
      // ttsannflite will startup even if configuration not there!
      //	return -1;
      WARN("AnnouncePath = \"%s\"\n", AnnouncePath.c_str());
      WARN("AnnounceFile = \"%s\"\n", AnnounceFile.c_str());
      WARN("Caching = true\n");
      WARN("CachePath = \"%s\"\n", CachePath.c_str());
    } else {
      if( ((p = m_config.getValueForKey("announce_path")) != NULL) && (*p != '\0') )
	AnnouncePath = p;
      else {
	WARN("no announce_path specified in configuration\n");
	WARN("file for module ttsannflitement.\n");
	AnnouncePath = DEFAULT_ANNOUNCE_PATH;
      }
      if( !AnnouncePath.empty() 
	  && AnnouncePath[AnnouncePath.length()-1] != '/' )
	AnnouncePath += "/";
 
      if( ((p = m_config.getValueForKey("default_announce")) != NULL) && (*p != '\0') )
	AnnounceFile = p;
      else {
	WARN("no default_announce specified in configuration\n");
	WARN("file for module ttsannflitement.\n");
	AnnounceFile = DEFAULT_ANNOUNCE_FILE;
      }

      string announce_file = AnnouncePath + AnnounceFile;
      if(!file_exists(announce_file)){
	ERROR("default file for ttsannflitement module does not exist ('%s').\n",
	      announce_file.c_str());
	//	return -1;
      }

      if( ((p = m_config.getValueForKey("caching")) != NULL) && (*p != '\0') ) {
	Caching = ((*p=='y') || (*p=='Y'));
      } else {
	WARN("no caching (y/n) specified in configuration\n");
	WARN("file for module ttsannflitement.\n");
	Caching = true;
      }

      if( ((p = m_config.getValueForKey("cache_path")) != NULL) && (*p != '\0') )
	CachePath = p;
      else {
	WARN("no cache_path specified in configuration\n");
	WARN("file for module ttsannflitement.\n");
	CachePath = DEFAULT_CACHE_PATH;
      }
      if( !CachePath.empty() 
	  && CachePath[CachePath.length()-1] != '/' )
	CachePath += "/";
    }
    return 0;
}

AmDialogState* TTSAnnfliteFactory::onInvite(AmCmd& cmd)
{
    string announce_path = AnnouncePath;
    string announce_file = announce_path + cmd.domain 
	+ "/" + cmd.user + ".txt";

    DBG("trying '%s'\n",announce_file.c_str());
    if(file_exists(announce_file))
	goto end;

    announce_file = announce_path + cmd.user + ".txt";
    DBG("trying '%s'\n",announce_file.c_str());
    if(file_exists(announce_file))
	goto end;

    announce_file = AnnouncePath + AnnounceFile;
    
end:
    return new TTSAnnfliteDialog(announce_file, Caching, CachePath);
}

TTSAnnfliteDialog::TTSAnnfliteDialog(const string& filename, 
				     const bool& Caching, const string& CachePath)
  : filename(filename), caching(Caching), cache_path(CachePath)
{
    flite_init();
    v = register_cmu_us_kal();
}

TTSAnnfliteDialog::~TTSAnnfliteDialog()
{
}

void TTSAnnfliteDialog::onSessionStart(AmRequest* req)
{
  try {
    DBG("starting sending ttsannflite...(%s)\n",filename.c_str());
    string msg_filename = string("/tmp/") + req->cmd.callid + string(".wav");
    int length;
    char *text;
      
    ifstream is;
    is.open (filename.c_str(), ios::binary );
      
    is.seekg (0, ios::end);
    length = is.tellg();
    is.seekg (0, ios::beg);
      
    text = new char [length+1];
    is.read (text,length);
    is.close();
    text[length]=0;
    string message(text);
      
    DBG("Message is :\"%s\"\n", message.c_str());

    string cache_filename = cache_path + string(text) + string(".wav");
    if (caching) {
      DBG(" trying cache \"%s\" .. ", cache_filename.c_str());
      if (file_exists(cache_filename)) {
	DBG("hit. Playing from cache.\n");
	getSession()->rtp_str.play(cache_filename.c_str());
	if(!getSession()->sess_stopped.get())
	  req->bye();
	return;
      } else {
	DBG("miss.\n");
      }
    }

    cst_wave *w;
    cst_utterance *u;
    u = flite_synth_text(message.c_str(), v);
    w = utt_wave(u);
    float durs;
    durs = (float)w->num_samples/(float)w->sample_rate;
	
    cst_wave_save_riff(w,msg_filename.c_str());
    if (caching)
      cst_wave_save_riff(w, cache_filename.c_str());
    delete_utterance(u);
    DBG("%f seconds of speech synthesized\n",durs);
    //-----------------

    getSession()->rtp_str.play(msg_filename.c_str());
    if(!getSession()->sess_stopped.get()) {
      DBG("sending bye...");
      req->bye();
    }
    DBG("done.");
  }
  catch(const string& err){
    ERROR("%s\n",err.c_str());
  }
}

void TTSAnnfliteDialog::onBye(AmRequest* req)
{
}

