/*
 * $Id: AnswerMachine.h,v 1.9 2004/06/16 12:30:46 rco Exp $
 *
 * Copyright (C) 2002-2003 Fhg Fokus
 *
 * This file is part of sems, a free SIP media server.
 *
 * sems is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * For a license to use the sems software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * sems is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _ANSWERMACHINE_H_
#define _ANSWERMACHINE_H_

#include "AmApi.h"
#include "SemsConfiguration.h"
#include "EmailTemplate.h"
#include "AmMail.h"

#include <string>
using std::string;

/* how many seconds to wait until call is accepted with 200 */
#define DEFAULT_ACCEPT_DELAY 0

class AmMail;

class AnswerMachineFactory: public AmStateFactory
{
    SemsConfiguration m_config;
    EmailTemplate email_tmpl;

public:
    static string RecFileExt;
    static string AnnouncePath;
    static string DefaultAnnounce;
    static int    MaxRecordTime;
    static int    AcceptDelay;

    AnswerMachineFactory(const string& _app_name);

    int onLoad();
    AmDialogState* onInvite(AmCmd&);

    // throws error string (see EmailTemplate::getEmail)
    AmMail getEmail(const AmCmd& cmd) {
	return email_tmpl.getEmail(cmd); 
    }
};

class AnswerMachineDialog : public AmDialogState
{
    string announce_file;
    const EmailTemplate* tmpl;

 public:
    AnswerMachineDialog(string announce_file, const EmailTemplate* tmpl);
    ~AnswerMachineDialog();

    void onSessionStart(AmRequest* req);
    void onBeforeCallAccept(AmRequest* req);
    void onBye(AmRequest* req);
    static void clean_up_mail(AmMail* mail);

    friend class AnswerMachineFactory;
};

#endif
// Local Variables:
// mode:C++
// End:

